@@ -14,18 +14,16 @@
 
 """The Scheduler Stats extension"""
 
+from cinder.api import common
 from cinder.api import extensions
+from cinder.api import microversions as mv
 from cinder.api.openstack import wsgi
 from cinder.api.views import scheduler_stats as scheduler_stats_view
+from cinder.policies import scheduler_stats as policy
 from cinder.scheduler import rpcapi
 from cinder import utils
 
 
-def authorize(context, action_name):
-    action = 'scheduler_stats:%s' % action_name
-    extensions.extension_authorizer('scheduler', action)(context)
-
-
 class SchedulerStatsController(wsgi.Controller):
     """The Scheduler Stats controller for the OpenStack API."""
 
@@ -35,14 +33,31 @@ class SchedulerStatsController(wsgi.Controller):
         self.scheduler_api = rpcapi.SchedulerAPI()
         super(SchedulerStatsController, self).__init__()
 
+    @common.process_general_filtering('pool')
+    def _process_pool_filtering(self, context=None, filters=None,
+                                req_version=None):
+        if not req_version.matches(mv.POOL_FILTER):
+            filters.clear()
+
     def get_pools(self, req):
         """List all active pools in scheduler."""
         context = req.environ['cinder.context']
-        authorize(context, 'get_pools')
+        context.authorize(policy.GET_POOL_POLICY)
 
-        # TODO(zhiteng) Add filters support
         detail = utils.get_bool_param('detail', req.params)
-        pools = self.scheduler_api.get_pools(context, filters=None)
+
+        req_version = req.api_version_request
+        filters = req.params.copy()
+        filters.pop('detail', None)
+
+        self._process_pool_filtering(context=context,
+                                     filters=filters,
+                                     req_version=req_version)
+
+        if not req_version.matches(mv.POOL_TYPE_FILTER):
+            filters.pop('volume_type', None)
+
+        pools = self.scheduler_api.get_pools(context, filters=filters)
 
         return self._view_builder.pools(req, pools, detail)
 
