@@ -13,8 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-
 from cinder.api import common
 from cinder import group as group_api
 from cinder.objects import fields
@@ -77,13 +75,13 @@ class ViewBuilder(common.ViewBuilder):
                 'updated_at': volume.get('updated_at'),
                 'name': volume.get('display_name'),
                 'description': volume.get('display_description'),
-                'volume_type': self._get_volume_type(volume),
+                'volume_type': self._get_volume_type(request, volume),
                 'snapshot_id': volume.get('snapshot_id'),
                 'source_volid': volume.get('source_volid'),
                 'metadata': self._get_volume_metadata(volume),
                 'links': self._get_links(request, volume['id']),
                 'user_id': volume.get('user_id'),
-                'bootable': six.text_type(volume.get('bootable')).lower(),
+                'bootable': str(volume.get('bootable')).lower(),
                 'encrypted': self._is_volume_encrypted(volume),
                 'replication_status': volume.get('replication_status'),
                 'consistencygroup_id': volume.get('consistencygroup_id'),
@@ -140,8 +138,8 @@ class ViewBuilder(common.ViewBuilder):
         """Retrieve the metadata of the volume object."""
         return volume.metadata
 
-    def _get_volume_type(self, volume):
-        """Retrieve the type the volume object."""
+    def _get_volume_type(self, request, volume):
+        """Retrieve the type of the volume object."""
         if volume['volume_type_id'] and volume.get('volume_type'):
             return volume['volume_type']['name']
         else:
