@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from cinder.api import microversions as mv
 from cinder.api.views import snapshots as views_v2
 
 
@@ -25,9 +26,12 @@ class ViewBuilder(views_v2.ViewBuilder):
 
         req_version = request.api_version_request
         # Add group_snapshot_id if min version is greater than or equal
-        # to 3.14.
-        if req_version.matches("3.14", None):
-            snapshot_ref['snapshot']['group_snapshot_id'] = (
-                snapshot.get('group_snapshot_id'))
-
+        # to GROUP_SNAPSHOTS.
+        snap = snapshot_ref['snapshot']
+        if req_version.matches(mv.GROUP_SNAPSHOTS, None):
+            snap['group_snapshot_id'] = snapshot.get('group_snapshot_id')
+        if req_version.matches(mv.SNAPSHOT_LIST_USER_ID, None):
+            snap['user_id'] = snapshot.get('user_id')
+        if req_version.matches(mv.USE_QUOTA):
+            snap['consumes_quota'] = snapshot.get('use_quota')
         return snapshot_ref
