@@ -26,11 +26,11 @@ from oslo_utils import timeutils
 import sqlalchemy as sa
 # imports needed for cinderlib
 from sqlalchemy import Column, String, Text  # noqa: F401
-from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import func
 from sqlalchemy import schema
 from sqlalchemy.sql import expression
-from sqlalchemy.orm import backref, column_property, relationship, validates
+from sqlalchemy.orm import backref, column_property, declarative_base, \
+    relationship, validates
 
 
 CONF = cfg.CONF
@@ -218,6 +218,11 @@ class Group(BASE, CinderBase):
     """Represents a generic volume group."""
 
     __tablename__ = 'groups'
+    __table_args__ = (
+        # Speed up normal listings
+        sa.Index('groups_deleted_project_id_idx', 'deleted', 'project_id'),
+        CinderBase.__table_args__,
+    )
 
     id = sa.Column(sa.String(36), primary_key=True)
 
@@ -272,6 +277,12 @@ class GroupSnapshot(BASE, CinderBase):
     """Represents a group snapshot."""
 
     __tablename__ = 'group_snapshots'
+    __table_args__ = (
+        # Speed up normal listings
+        sa.Index('group_snapshots_deleted_project_id_idx',
+                 'deleted', 'project_id'),
+        CinderBase.__table_args__,
+    )
 
     id = sa.Column(sa.String(36), primary_key=True)
 
@@ -303,17 +314,22 @@ class Volume(BASE, CinderBase):
 
     __tablename__ = 'volumes'
     __table_args__ = (
-        sa.Index('volumes_service_uuid_idx', 'deleted', 'service_uuid'),
+        sa.Index('volumes_service_uuid_idx', 'service_uuid', 'deleted'),
+        # Speed up normal listings
+        sa.Index('volumes_deleted_project_id_idx', 'deleted', 'project_id'),
+        # Speed up service start, create volume from image when using direct
+        # urls, host REST API, and the cinder-manage update host cmd
+        sa.Index('volumes_deleted_host_idx', 'deleted', 'host'),
         CinderBase.__table_args__,
     )
 
     id = sa.Column(sa.String(36), primary_key=True)
     _name_id = sa.Column(sa.String(36))  # Don't access/modify this directly!
-    # TODO: (Y release) Change nullable to False
     use_quota = Column(
         sa.Boolean,
-        nullable=True,
+        nullable=False,
         default=True,
+        server_default=sa.true(),
         doc='Ignore volume in quota usage',
     )
 
@@ -627,7 +643,10 @@ class VolumeTypeExtraSpecs(BASE, CinderBase):
     value = sa.Column(sa.String(255))
     volume_type_id = sa.Column(
         sa.String(36),
-        sa.ForeignKey('volume_types.id'),
+        sa.ForeignKey(
+            'volume_types.id',
+            name='volume_type_extra_specs_ibfk_1',
+        ),
         nullable=False,
         index=True,
     )
@@ -786,10 +805,8 @@ class Quota(BASE, CinderBase):
     # TODO(stephenfin): Add index=True
     project_id = sa.Column(sa.String(255))
 
-    resource = sa.Column(sa.String(255), nullable=False)
+    resource = sa.Column(sa.String(300), nullable=False)
     hard_limit = sa.Column(sa.Integer, nullable=True)
-    # TODO: (X release): Remove allocated, belonged to nested quotas
-    allocated = sa.Column(sa.Integer, default=0)
 
 
 class QuotaClass(BASE, CinderBase):
@@ -806,7 +823,7 @@ class QuotaClass(BASE, CinderBase):
 
     class_name = sa.Column(sa.String(255), index=True)
 
-    resource = sa.Column(sa.String(255))
+    resource = sa.Column(sa.String(300))
     hard_limit = sa.Column(sa.Integer, nullable=True)
 
 
@@ -864,13 +881,9 @@ class Reservation(BASE, CinderBase):
     usage_id = sa.Column(
         sa.Integer, sa.ForeignKey('quota_usages.id'), nullable=True, index=True
     )
-    # TODO: (X release): Remove allocated_id, belonged to nested quotas
-    allocated_id = sa.Column(
-        sa.Integer, sa.ForeignKey('quotas.id'), nullable=True, index=True
-    )
 
     project_id = sa.Column(sa.String(255), index=True)
-    resource = sa.Column(sa.String(255))
+    resource = sa.Column(sa.String(300))
 
     delta = sa.Column(sa.Integer, nullable=False)
     # TODO(stephenfin): Add nullable=False
@@ -882,25 +895,24 @@ class Reservation(BASE, CinderBase):
         primaryjoin='and_(Reservation.usage_id == QuotaUsage.id,'
         'QuotaUsage.deleted == False)',
     )
-    # TODO: (X release): Remove allocated_id, belonged to nested quotas
-    quota = relationship(
-        "Quota",
-        foreign_keys=allocated_id,
-        primaryjoin='and_(Reservation.allocated_id == Quota.id)',
-    )
 
 
 class Snapshot(BASE, CinderBase):
     """Represents a snapshot of volume."""
 
     __tablename__ = 'snapshots'
+    __table_args__ = (
+        # Speed up normal listings
+        sa.Index('snapshots_deleted_project_id_idx', 'deleted', 'project_id'),
+        CinderBase.__table_args__,
+    )
 
     id = sa.Column(sa.String(36), primary_key=True)
-    # TODO: (Y release) Change nullable to False
     use_quota = Column(
         sa.Boolean,
-        nullable=True,
+        nullable=False,
         default=True,
+        server_default=sa.true(),
         doc='Ignore volume in quota usage',
     )
 
@@ -996,6 +1008,11 @@ class Backup(BASE, CinderBase):
     """Represents a backup of a volume to Swift."""
 
     __tablename__ = 'backups'
+    __table_args__ = (
+        # Speed up normal listings
+        sa.Index('backups_deleted_project_id_idx', 'deleted', 'project_id'),
+        CinderBase.__table_args__,
+    )
 
     id = sa.Column(sa.String(36), primary_key=True)
     # Backups don't have use_quota field since we don't have temporary backups
@@ -1202,9 +1219,9 @@ class Worker(BASE, CinderBase):
 
     # Type of the resource we are working on (Volume, Snapshot, Backup) it must
     # match the Versioned Object class name.
-    resource_type = sa.Column(sa.String(40), primary_key=True, nullable=False)
+    resource_type = sa.Column(sa.String(40), nullable=False)
     # UUID of the resource we are working on
-    resource_id = sa.Column(sa.String(36), primary_key=True, nullable=False)
+    resource_id = sa.Column(sa.String(36), nullable=False)
 
     # Status that should be cleaned on service failure
     status = sa.Column(sa.String(255), nullable=False)
