@@ -24,21 +24,6 @@ from cinder import exception
 import cinder.privsep
 
 
-@cinder.privsep.sys_admin_pctxt.entrypoint
-def readfile(path):
-    if not os.path.exists(path):
-        raise exception.FileNotFound(file_path=path)
-    with open(path, 'r') as f:
-        return f.read()
-
-
-@cinder.privsep.sys_admin_pctxt.entrypoint
-def removefile(path):
-    if not os.path.exists(path):
-        raise exception.FileNotFound(file_path=path)
-    os.unlink(path)
-
-
 @cinder.privsep.sys_admin_pctxt.entrypoint
 def touch(path):
     if os.path.exists(path):
