@@ -42,12 +42,13 @@ CONF.register_opts(scheduler_driver_opts)
 
 
 def volume_update_db(context, volume_id, host, cluster_name,
-                     availability_zone=None):
+                     availability_zone=None, volume=None):
     """Set the host, cluster_name, and set the scheduled_at field of a volume.
 
     :returns: A Volume with the updated fields set properly.
     """
-    volume = objects.Volume.get_by_id(context, volume_id)
+    if not volume:
+        volume = objects.Volume.get_by_id(context, volume_id)
     volume.host = host
     volume.cluster_name = cluster_name
     volume.scheduled_at = timeutils.utcnow()
@@ -155,7 +156,7 @@ class Scheduler(object):
         raise NotImplementedError(_(
             "Must implement schedule_get_pools"))
 
-    def get_backup_host(self, volume, driver=None):
+    def get_backup_host(self, volume, availability_zone, driver=None):
         """Must override schedule method for scheduler to work."""
         raise NotImplementedError(_(
             "Must implement get_backup_host"))
