@@ -12,8 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from http import HTTPStatus
+
 from oslo_middleware import request_id
-from six.moves import http_client
 import webob
 
 import cinder.api.middleware.auth
@@ -39,32 +40,32 @@ class TestCinderKeystoneContextMiddleware(test.TestCase):
 
     def test_no_user_or_user_id(self):
         response = self.request.get_response(self.middleware)
-        self.assertEqual(http_client.UNAUTHORIZED, response.status_int)
+        self.assertEqual(HTTPStatus.UNAUTHORIZED, response.status_int)
 
     def test_user_only(self):
         self.request.headers['X_USER'] = 'testuser'
         response = self.request.get_response(self.middleware)
-        self.assertEqual(http_client.OK, response.status_int)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
         self.assertEqual('testuser', self.context.user_id)
 
     def test_user_id_only(self):
         self.request.headers['X_USER_ID'] = 'testuserid'
         response = self.request.get_response(self.middleware)
-        self.assertEqual(http_client.OK, response.status_int)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
         self.assertEqual('testuserid', self.context.user_id)
 
     def test_user_id_trumps_user(self):
         self.request.headers['X_USER_ID'] = 'testuserid'
         self.request.headers['X_USER'] = 'testuser'
         response = self.request.get_response(self.middleware)
-        self.assertEqual(http_client.OK, response.status_int)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
         self.assertEqual('testuserid', self.context.user_id)
 
     def test_tenant_id_name(self):
         self.request.headers['X_USER_ID'] = 'testuserid'
         self.request.headers['X_TENANT_NAME'] = 'testtenantname'
         response = self.request.get_response(self.middleware)
-        self.assertEqual(http_client.OK, response.status_int)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
         self.assertEqual('testtenantid', self.context.project_id)
         self.assertEqual('testtenantname', self.context.project_name)
 
