@@ -16,9 +16,9 @@
 """
 Tests for group API.
 """
+from unittest import mock
 
 import ddt
-import mock
 
 from cinder import context
 from cinder import exception
@@ -27,9 +27,9 @@ from cinder import objects
 from cinder.objects import fields
 from cinder.policies import group_snapshots as g_snap_policies
 from cinder import quota
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder.tests.unit import utils
 
 
@@ -274,6 +274,8 @@ class GroupAPITestCase(test.TestCase):
                           self.group_api._validate_add_volumes, self.ctxt,
                           [], ['123456789'], grp)
 
+    @ddt.data(['test_host@fakedrv#fakepool', 'test_host@fakedrv#fakepool'],
+              ['test_host@fakedrv#fakepool', 'test_host2@fakedrv#fakepool'])
     @mock.patch('cinder.volume.rpcapi.VolumeAPI.update_group')
     @mock.patch('cinder.db.volume_get_all_by_generic_group')
     @mock.patch('cinder.group.api.API._cast_create_group')
@@ -281,7 +283,7 @@ class GroupAPITestCase(test.TestCase):
     @mock.patch('cinder.objects.Group')
     @mock.patch('cinder.db.group_type_get')
     @mock.patch('cinder.db.volume_types_get_by_name_or_id')
-    def test_update(self, mock_volume_types_get,
+    def test_update(self, hosts, mock_volume_types_get,
                     mock_group_type_get, mock_group,
                     mock_update_quota, mock_cast_create_group,
                     mock_volume_get_all, mock_rpc_update_group):
@@ -307,26 +309,32 @@ class GroupAPITestCase(test.TestCase):
         self.assertEqual(grp.obj_to_primitive(), ret_group.obj_to_primitive())
 
         ret_group.volume_types = [vol_type]
-        ret_group.host = "test_host@fakedrv#fakepool"
+        ret_group.host = hosts[0]
+        # set resource_backend directly because ret_group
+        # is instance of MagicMock
+        ret_group.resource_backend = 'fake-cluster'
         ret_group.status = fields.GroupStatus.AVAILABLE
+
         ret_group.id = fake.GROUP_ID
 
         vol1 = utils.create_volume(
-            self.ctxt, host=ret_group.host,
-            availability_zone=ret_group.availability_zone,
-            volume_type_id=fake.VOLUME_TYPE_ID)
-
-        vol2 = utils.create_volume(
-            self.ctxt, host=ret_group.host,
+            self.ctxt, host=hosts[1],
             availability_zone=ret_group.availability_zone,
             volume_type_id=fake.VOLUME_TYPE_ID,
-            group_id=fake.GROUP_ID)
+            cluster_name='fake-cluster')
+
+        vol2 = utils.create_volume(
+            self.ctxt, host=hosts[1],
+            availability_zone=ret_group.availability_zone,
+            volume_type_id=fake.VOLUME_TYPE_ID,
+            group_id=fake.GROUP_ID,
+            cluster_name='fake-cluster')
         vol2_dict = {
             'id': vol2.id,
             'group_id': fake.GROUP_ID,
             'volume_type_id': fake.VOLUME_TYPE_ID,
             'availability_zone': ret_group.availability_zone,
-            'host': ret_group.host,
+            'host': hosts[1],
             'status': 'available',
         }
         mock_volume_get_all.return_value = [vol2_dict]
@@ -500,6 +508,7 @@ class GroupAPITestCase(test.TestCase):
         vol1.destroy()
         grp_snap.destroy()
 
+    @mock.patch('cinder.group.api.API._update_volumes_host')
     @mock.patch('cinder.objects.VolumeType.get_by_name_or_id')
     @mock.patch('cinder.db.group_volume_type_mapping_create')
     @mock.patch('cinder.volume.api.API.create')
@@ -512,7 +521,8 @@ class GroupAPITestCase(test.TestCase):
                                     mock_snap_get_all, mock_group_snap_get,
                                     mock_volume_api_create,
                                     mock_mapping_create,
-                                    mock_get_volume_type):
+                                    mock_get_volume_type,
+                                    mock_update_volumes_host):
         vol_type = fake_volume.fake_volume_type_obj(
             self.ctxt,
             id=fake.VOLUME_TYPE_ID,
@@ -566,12 +576,17 @@ class GroupAPITestCase(test.TestCase):
         mock_rpc_create_group_from_src.assert_called_once_with(
             self.ctxt, grp, grp_snap)
 
+        mock_update_volumes_host.assert_called_once_with(
+            self.ctxt, grp
+        )
+
         vol2.destroy()
         grp.destroy()
         snap.destroy()
         vol1.destroy()
         grp_snap.destroy()
 
+    @mock.patch('cinder.group.api.API._update_volumes_host')
     @mock.patch('cinder.objects.VolumeType.get_by_name_or_id')
     @mock.patch('cinder.db.group_volume_type_mapping_create')
     @mock.patch('cinder.volume.api.API.create')
@@ -583,7 +598,8 @@ class GroupAPITestCase(test.TestCase):
                                      mock_group_get,
                                      mock_volume_api_create,
                                      mock_mapping_create,
-                                     mock_get_volume_type):
+                                     mock_get_volume_type,
+                                     mock_update_volumes_host):
         vol_type = fake_volume.fake_volume_type_obj(
             self.ctxt,
             id=fake.VOLUME_TYPE_ID,
@@ -631,6 +647,10 @@ class GroupAPITestCase(test.TestCase):
         mock_rpc_create_group_from_src.assert_called_once_with(
             self.ctxt, grp2, None, grp)
 
+        mock_update_volumes_host.assert_called_once_with(
+            self.ctxt, grp2
+        )
+
         vol2.destroy()
         grp2.destroy()
         vol.destroy()
@@ -781,6 +801,42 @@ class GroupAPITestCase(test.TestCase):
                           self.ctxt, 'group', 'desc',
                           group_snapshot_id=None, source_group_id=group.id)
 
+    @mock.patch('cinder.objects.volume.Volume.host',
+                new_callable=mock.PropertyMock)
+    @mock.patch('cinder.objects.volume.Volume.cluster_name',
+                new_callable=mock.PropertyMock)
+    @mock.patch('cinder.objects.VolumeList.get_all_by_generic_group')
+    def test_update_volumes_host(self, mock_volume_get_all, mock_cluster_name,
+                                 mock_host):
+        vol_type = utils.create_volume_type(self.ctxt, name='test_vol_type')
+        grp = utils.create_group(self.ctxt, group_type_id=fake.GROUP_TYPE_ID,
+                                 volume_type_ids=[vol_type['id']],
+                                 availability_zone='nova',
+                                 status=fields.GroupStatus.CREATING,
+                                 cluster_name='fake_cluster')
+
+        vol1 = utils.create_volume(
+            self.ctxt,
+            availability_zone=grp.availability_zone,
+            volume_type_id=fake.VOLUME_TYPE_ID,
+            group_id=grp.id)
+
+        mock_volume = mock.Mock()
+        mock_volume_get_all.return_value = [mock_volume]
+        group_api = cinder.group.api.API()
+        group_api._update_volumes_host(None, grp)
+
+        mock_cluster_name.assert_called()
+        mock_host.assert_called()
+
+        self.assertEqual(grp.host, mock_volume.host)
+        self.assertEqual(grp.cluster_name, mock_volume.cluster_name)
+        mock_volume.save.assert_called_once_with()
+
+        vol1.destroy()
+
+        grp.destroy()
+
     def test_delete_group_frozen(self):
         service = utils.create_service(self.ctxt, {'frozen': True})
         group = utils.create_group(self.ctxt, host=service.host,
