@@ -21,10 +21,10 @@ from unittest import mock
 
 import ddt
 import glanceclient.exc
+from keystoneauth1 import loading as ksloading
 from keystoneauth1.loading import session as ks_session
 from keystoneauth1 import session
 from oslo_config import cfg
-import six
 
 from cinder import context
 from cinder import exception
@@ -113,7 +113,8 @@ class TestGlanceImageService(test.TestCase):
         self.mock_object(glance.time, 'sleep', return_value=None)
 
     def _create_image_service(self, client):
-        def _fake_create_glance_client(context, netloc, use_ssl):
+        def _fake_create_glance_client(
+                context, netloc, use_ssl, privileged_user=False):
             return client
 
         self.mock_object(glance, '_create_glance_client',
@@ -774,6 +775,99 @@ class TestGlanceImageService(test.TestCase):
         self.assertEqual(self.NOW_DATETIME, image_meta['created_at'])
         self.assertEqual(self.NOW_DATETIME, image_meta['updated_at'])
 
+    @mock.patch.object(glance.GlanceClientWrapper, 'call')
+    def test_add_location(self, mock_call):
+        image_id = mock.sentinel.image_id
+        service = glance.GlanceImageService(client=mock_call)
+        url = 'cinder://fake-store/c984be2b-8789-4b9e-bf71-19164f537e63'
+        metadata = {'store': 'fake-store'}
+
+        service.add_location(self.context, image_id, url, metadata)
+        mock_call.assert_called_once_with(
+            self.context, 'add_image_location',
+            image_id, url, metadata, privileged_user=True)
+
+    @mock.patch.object(glance.GlanceClientWrapper, 'call')
+    def test_add_location_old(self, mock_call):
+        mock_call.side_effect = [glanceclient.exc.HTTPNotImplemented, None]
+        image_id = mock.sentinel.image_id
+        service = glance.GlanceImageService(client=mock_call)
+        url = 'cinder://fake-store/c984be2b-8789-4b9e-bf71-19164f537e63'
+        metadata = {'store': 'fake-store'}
+
+        service.add_location(self.context, image_id, url, metadata)
+        calls = [
+            mock.call.call(
+                self.context, 'add_image_location',
+                image_id, url, metadata, privileged_user=True),
+            mock.call.call(
+                self.context, 'add_location',
+                image_id, url, metadata, privileged_user=False)]
+        mock_call.assert_has_calls(calls)
+
+    @mock.patch.object(glance.GlanceClientWrapper, 'call')
+    def test_get_location(self, mock_call):
+        url = 'cinder://fake-store/c984be2b-8789-4b9e-bf71-19164f537e63'
+        meta = {'store': 'fake-store'}
+        locations = [{'url': url, 'metadata': meta}]
+        mock_call.return_value = iter(locations)
+        image_id = mock.sentinel.image_id
+        service = glance.GlanceImageService(client=mock_call)
+
+        direct_url, locations = service.get_location(self.context, image_id)
+        mock_call.assert_called_once_with(self.context, 'get_image_locations',
+                                          image_id, privileged_user=True)
+        self.assertIsNone(direct_url)
+        self.assertEqual(1, len(locations))
+        self.assertEqual(url, locations[0]['url'])
+        self.assertEqual(meta, locations[0]['metadata'])
+
+    @mock.patch.object(glance.GlanceClientWrapper, 'call')
+    def test_get_location_old_not_implemented(self, mock_call):
+        class ImageMeta:
+            def __init__(self, image_meta):
+                self.__dict__.update(image_meta)
+        url = 'cinder://fake-store/c984be2b-8789-4b9e-bf71-19164f537e63'
+        meta = {'store': 'fake-store'}
+        loc = {'url': url, 'metadata': meta}
+        get_loc = ImageMeta({'direct_url': url, 'locations': loc})
+        mock_call.side_effect = [glanceclient.exc.HTTPNotImplemented, get_loc]
+        image_id = mock.sentinel.image_id
+        service = glance.GlanceImageService(client=mock_call)
+
+        direct_url, locations = service.get_location(self.context, image_id)
+        calls = [
+            mock.call.call(self.context, 'get_image_locations',
+                           image_id, privileged_user=True),
+            mock.call.call(self.context, 'get', image_id,
+                           privileged_user=False)]
+        mock_call.assert_has_calls(calls)
+        self.assertEqual(url, direct_url)
+        self.assertEqual(loc, locations)
+
+    @mock.patch.object(glance.GlanceClientWrapper, 'call')
+    def test_get_location_old_forbidden(self, mock_call):
+        class ImageMeta:
+            def __init__(self, image_meta):
+                self.__dict__.update(image_meta)
+        url = 'cinder://fake-store/c984be2b-8789-4b9e-bf71-19164f537e63'
+        meta = {'store': 'fake-store'}
+        loc = {'url': url, 'metadata': meta}
+        get_loc = ImageMeta({'direct_url': url, 'locations': loc})
+        mock_call.side_effect = [glanceclient.exc.HTTPForbidden, get_loc]
+        image_id = mock.sentinel.image_id
+        service = glance.GlanceImageService(client=mock_call)
+
+        direct_url, locations = service.get_location(self.context, image_id)
+        calls = [
+            mock.call.call(self.context, 'get_image_locations',
+                           image_id, privileged_user=True),
+            mock.call.call(self.context, 'get', image_id,
+                           privileged_user=False)]
+        mock_call.assert_has_calls(calls)
+        self.assertEqual(url, direct_url)
+        self.assertEqual(loc, locations)
+
     def test_download_with_retries(self):
         tries = [0]
 
@@ -815,8 +909,8 @@ class TestGlanceImageService(test.TestCase):
         image_id = 'fake-image-uuid'
         e = self.assertRaises(exception.ImageDownloadFailed, service.download,
                               self.context, image_id)
-        self.assertIn('image contains no data', six.text_type(e))
-        self.assertIn(image_id, six.text_type(e))
+        self.assertIn('image contains no data', str(e))
+        self.assertIn(image_id, str(e))
 
     def test_client_forbidden_converts_to_imagenotauthed(self):
         class MyGlanceStubClient(glance_stubs.StubGlanceClient):
@@ -926,7 +1020,7 @@ class TestGlanceImageService(test.TestCase):
         self.assertRaises(exception.ImageNotFound, service.download,
                           self.context, image_id, writer)
 
-    @mock.patch('six.moves.builtins.open')
+    @mock.patch('builtins.open', new_callable=mock.mock_open)
     @mock.patch('shutil.copyfileobj')
     @mock.patch('cinder.image.glance.get_api_servers',
                 return_value=itertools.cycle([(False, 'localhost:9292')]))
@@ -941,7 +1035,7 @@ class TestGlanceImageService(test.TestCase):
         mock_copyfileobj.assert_called_once_with(mock.ANY, writer)
         mock_open.assert_called_once_with('/tmp/test', 'rb')
 
-    @mock.patch('six.moves.builtins.open')
+    @mock.patch('builtins.open', new_callable=mock.mock_open)
     @mock.patch('shutil.copyfileobj')
     @mock.patch('cinder.image.glance.get_api_servers',
                 return_value=itertools.cycle([(False, 'localhost:9292')]))
@@ -1252,7 +1346,7 @@ class TestGlanceImageServiceClient(test.TestCase):
         client = glance._create_glance_client(self.context, 'fake_host:9292',
                                               False)
         self.assertIsInstance(client, MyGlanceStubClient)
-        mock_get_auth_plugin.assert_called_once_with(self.context)
+        mock_get_auth_plugin.assert_called_once_with(self.context, auth=None)
         mock_load.assert_called_once_with(**config_options)
 
     @mock.patch('cinder.service_auth.get_auth_plugin')
@@ -1288,7 +1382,7 @@ class TestGlanceImageServiceClient(test.TestCase):
         client = glance._create_glance_client(self.context, 'fake_host:9292',
                                               True)
         self.assertIsInstance(client, MyGlanceStubClient)
-        mock_get_auth_plugin.assert_called_once_with(self.context)
+        mock_get_auth_plugin.assert_called_once_with(self.context, auth=None)
         mock_load.assert_called_once_with(**config_options)
 
     def test_create_glance_client_auth_strategy_noauth_with_protocol_https(
@@ -1332,3 +1426,15 @@ class TestGlanceImageServiceClient(test.TestCase):
         client = glance._create_glance_client(self.context, 'fake_host:9292',
                                               False)
         self.assertIsInstance(client, MyGlanceStubClient)
+
+    @mock.patch('cinder.service_auth.get_auth_plugin')
+    @mock.patch.object(ksloading, 'load_auth_from_conf_options')
+    def test_create_glance_client_with_privileged_user(
+            self, mock_load, mock_get_auth_plugin):
+        self.flags(auth_strategy='keystone')
+        self.flags(auth_type='password', group='glance')
+        mock_load.return_value = 'fake_auth_plugin'
+        glance.GlanceClientWrapper(self.context, 'fake_host:9292', True, True)
+        mock_load.assert_called_once()
+        mock_get_auth_plugin.assert_called_once_with(
+            self.context, auth='fake_auth_plugin')
