@@ -13,10 +13,9 @@
 # under the License.
 
 from unittest import mock
+from zoneinfo import ZoneInfo
 
 from oslo_utils import timeutils
-import pytz
-import six
 
 from cinder import exception
 from cinder import objects
@@ -195,8 +194,9 @@ class TestConsistencyGroup(test_objects.BaseObjectsTestCase):
         self.assertTrue(consistencygroup.deleted)
         self.assertEqual(fields.ConsistencyGroupStatus.DELETED,
                          consistencygroup.status)
-        self.assertEqual(utcnow_mock.return_value.replace(tzinfo=pytz.UTC),
-                         consistencygroup.deleted_at)
+        self.assertEqual(
+            utcnow_mock.return_value.replace(tzinfo=ZoneInfo('UTC')),
+            consistencygroup.deleted_at)
 
     @mock.patch('cinder.db.sqlalchemy.api.consistencygroup_get')
     def test_refresh(self, consistencygroup_get):
@@ -215,15 +215,11 @@ class TestConsistencyGroup(test_objects.BaseObjectsTestCase):
         # new value for that field
         cg.refresh()
         self._compare(self, db_cg2, cg)
-        if six.PY3:
-            call_bool = mock.call.__bool__()
-        else:
-            call_bool = mock.call.__nonzero__()
         consistencygroup_get.assert_has_calls([
             mock.call(
                 self.context,
                 fake.CONSISTENCY_GROUP_ID),
-            call_bool,
+            mock.call.__bool__(),
             mock.call(
                 self.context,
                 fake.CONSISTENCY_GROUP_ID)])
