@@ -27,6 +27,7 @@ from cinder.scheduler import filters
 from cinder.scheduler.filters import extra_specs_ops
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit.scheduler import fakes
+from cinder.tests.unit.scheduler import helpers
 from cinder.tests.unit import test
 from cinder.tests.unit import utils
 
@@ -37,13 +38,10 @@ class BackendFiltersTestCase(test.TestCase):
     def setUp(self):
         super(BackendFiltersTestCase, self).setUp()
         self.context = context.RequestContext(fake.USER_ID, fake.PROJECT_ID)
-        # This has a side effect of testing 'get_filter_classes'
-        # when specifying a method (in this case, our standard filters)
-        filter_handler = filters.BackendFilterHandler(
-            'cinder.scheduler.filters')
-        classes = filter_handler.get_all_classes()
+        # we're testing that the filters work, not that they are retrievable
+        # so pre-populate this convenient class map
         self.class_map = {}
-        for cls in classes:
+        for cls in helpers.ALL_FILTER_CLASSES:
             self.class_map[cls.__name__] = cls
 
 
@@ -99,7 +97,7 @@ class CapacityFilterTestCase(BackendFiltersTestCase):
     def test_filter_fails(self, _mock_serv_is_up):
         _mock_serv_is_up.return_value = True
         filt_cls = self.class_map['CapacityFilter']()
-        filter_properties = {'size': 100,
+        filter_properties = {'size': 121,
                              'request_spec': {'volume_id': fake.VOLUME_ID}}
         service = {'disabled': False}
         host = fakes.FakeBackendState('host1',
@@ -282,7 +280,7 @@ class CapacityFilterTestCase(BackendFiltersTestCase):
     def test_filter_thin_true_passes2(self, _mock_serv_is_up):
         _mock_serv_is_up.return_value = True
         filt_cls = self.class_map['CapacityFilter']()
-        filter_properties = {'size': 3000,
+        filter_properties = {'size': 2400,
                              'capabilities:thin_provisioning_support':
                                  '<is> True',
                              'capabilities:thick_provisioning_support':
@@ -462,7 +460,7 @@ class CapacityFilterTestCase(BackendFiltersTestCase):
     def test_filter_reserved_thin_thick_true_fails(self, _mock_serv_is_up):
         _mock_serv_is_up.return_value = True
         filt_cls = self.class_map['CapacityFilter']()
-        filter_properties = {'size': 100,
+        filter_properties = {'size': 151,
                              'capabilities:thin_provisioning_support':
                                  '<is> True',
                              'capabilities:thick_provisioning_support':
@@ -1025,7 +1023,7 @@ class InstanceLocalityFilterTestCase(BackendFiltersTestCase):
             fakes.FakeNovaClient().list_extensions.show_all())
         filt_cls = self.class_map['InstanceLocalityFilter']()
         host = fakes.FakeBackendState('host1', {})
-        uuid = nova.novaclient().servers.create('host1')
+        uuid = nova.novaclient(context=self.context).servers.create('host1')
 
         filter_properties = {'context': self.context,
                              'scheduler_hints': {'local_to_instance': uuid},
@@ -1040,7 +1038,7 @@ class InstanceLocalityFilterTestCase(BackendFiltersTestCase):
             fakes.FakeNovaClient().list_extensions.show_all())
         filt_cls = self.class_map['InstanceLocalityFilter']()
         host = fakes.FakeBackendState('host1', {})
-        uuid = nova.novaclient().servers.create('host2')
+        uuid = nova.novaclient(context=self.context).servers.create('host2')
 
         filter_properties = {'context': self.context,
                              'scheduler_hints': {'local_to_instance': uuid},
@@ -1067,20 +1065,6 @@ class InstanceLocalityFilterTestCase(BackendFiltersTestCase):
         self.assertRaises(exception.InvalidUUID,
                           filt_cls.backend_passes, host, filter_properties)
 
-    @mock.patch('cinder.compute.nova.novaclient')
-    def test_nova_no_extended_server_attributes(self, _mock_novaclient):
-        _mock_novaclient.return_value = fakes.FakeNovaClient(
-            ext_srv_attr=False)
-        filt_cls = self.class_map['InstanceLocalityFilter']()
-        host = fakes.FakeBackendState('host1', {})
-        uuid = nova.novaclient().servers.create('host1')
-
-        filter_properties = {'context': self.context,
-                             'scheduler_hints': {'local_to_instance': uuid},
-                             'request_spec': {'volume_id': fake.VOLUME_ID}}
-        self.assertRaises(exception.CinderException,
-                          filt_cls.backend_passes, host, filter_properties)
-
     @mock.patch('cinder.compute.nova.novaclient')
     def test_nova_down_does_not_alter_other_filters(self, _mock_novaclient):
         # Simulate Nova API is not available
@@ -1096,8 +1080,8 @@ class InstanceLocalityFilterTestCase(BackendFiltersTestCase):
     @mock.patch('cinder.compute.nova.novaclient')
     def test_nova_timeout(self, mock_novaclient):
         # Simulate a HTTP timeout
-        mock_show_all = mock_novaclient.return_value.list_extensions.show_all
-        mock_show_all.side_effect = request_exceptions.Timeout
+        mock_get = mock_novaclient.return_value.servers.get
+        mock_get.side_effect = request_exceptions.Timeout
 
         filt_cls = self.class_map['InstanceLocalityFilter']()
         host = fakes.FakeBackendState('host1', {})
