@@ -51,6 +51,8 @@ class MockConfig(object):
         self.driver_ssl_cert_verify = True
         self.driver_ssl_cert_path = None
         self.remove_empty_host = False
+        self.use_multipath_for_image_xfer = False
+        self.enforce_multipath_for_image_xfer = False
 
     def safe_get(self, name):
         return getattr(self, name)
@@ -391,7 +393,7 @@ def get_backend_qos_specs(volume):
     return None
 
 
-def get_connector_properties():
+def get_connector_properties(use_multipath, enforce_multipath):
     return {'host': 'host1', 'wwpns': 'abcdefg'}
 
 
@@ -470,7 +472,7 @@ def patch_for_unity_adapter(func):
     @mock.patch('cinder.volume.drivers.dell_emc.unity.utils.'
                 'group_is_cg',
                 new=group_is_cg)
-    @mock.patch('cinder.utils.brick_get_connector_properties',
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector_properties',
                 new=get_connector_properties)
     def func_wrapper(*args, **kwargs):
         return func(*args, **kwargs)
@@ -1847,8 +1849,8 @@ class ISCSIAdapterTest(test.TestCase):
         self.assertListEqual([hlu, hlu], info['target_luns'])
         self.assertListEqual(target_portals, info['target_portals'])
         self.assertEqual(hlu, info['target_lun'])
-        self.assertTrue(info['target_portal'] in target_portals)
-        self.assertTrue(info['target_iqn'] in target_iqns)
+        self.assertIn(info['target_portal'], target_portals)
+        self.assertIn(info['target_iqn'], target_iqns)
 
     @patch_for_iscsi_adapter
     def test_initialize_connection_volume(self):
