@@ -121,6 +121,51 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         mock_get_ontap_version.assert_called_once_with(cached=False)
         mock_get_cluster_nodes_info.assert_called_once_with()
 
+    @ddt.data(fake.AFF_SYSTEM_NODES_INFO,
+              fake.FAS_SYSTEM_NODES_INFO,
+              fake.HYBRID_SYSTEM_NODES_INFO)
+    @mock.patch.object(client_base.Client, 'get_ontap_version',
+                       return_value='9.6')
+    @mock.patch.object(perf_cmode, 'PerformanceCmodeLibrary', mock.Mock())
+    @mock.patch.object(client_base.Client, 'get_ontapi_version',
+                       mock.MagicMock(return_value=(1, 20)))
+    @mock.patch.object(capabilities.CapabilitiesLibrary,
+                       'cluster_user_supported')
+    @mock.patch.object(capabilities.CapabilitiesLibrary,
+                       'check_api_permissions')
+    @mock.patch.object(na_utils, 'check_flags')
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, 'do_setup')
+    def test_do_setup_with_replication(self, cluster_nodes_info,
+                                       super_do_setup, mock_check_flags,
+                                       mock_check_api_permissions,
+                                       mock_cluster_user_supported,
+                                       mock_get_ontap_version):
+        """Tests setup method when replication is enabled"""
+        self.mock_object(client_base.Client, '_init_ssh_client')
+        mock_get_cluster_nodes_info = self.mock_object(
+            client_cmode.Client, '_get_cluster_nodes_info',
+            return_value=cluster_nodes_info)
+        self.mock_object(
+            dot_utils, 'get_backend_configuration',
+            return_value=self.get_config_cmode())
+        context = mock.Mock()
+        self.replication_enabled = True
+        self.replication_policy = "AutomatedFailOver"
+        self.replication_backends = ['target_1', 'target_2']
+        self.mock_object(self.library, 'get_replication_backend_names',
+                         return_value=self.replication_backends)
+        self.mock_object(self.library, 'get_replication_policy',
+                         return_value=self.replication_policy)
+
+        self.library.do_setup(context)
+
+        super_do_setup.assert_called_once_with(context)
+        self.assertEqual(1, mock_check_flags.call_count)
+        mock_check_api_permissions.assert_called_once_with()
+        mock_cluster_user_supported.assert_called_once_with()
+        mock_get_ontap_version.assert_called_once_with(cached=False)
+        mock_get_cluster_nodes_info.assert_called_once_with()
+
     def test_check_for_setup_error(self):
         super_check_for_setup_error = self.mock_object(
             block_base.NetAppBlockStorageLibrary, 'check_for_setup_error')
@@ -334,6 +379,57 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             qos_policy_group_is_adaptive=False,
             source_snapshot=None, is_snapshot=True)
 
+    def test_clone_lun_busy_exception(self):
+        """Test for when clone lun is throwing device busy error."""
+        self.library._get_lun_attr = mock.Mock(
+            return_value={'Volume': 'fakeLUN'})
+        self.library.zapi_client = mock.Mock()
+        lun = fake.FAKE_LUN_GET_ITER_RESULT
+        self.library.zapi_client.get_lun_by_args.return_value = lun
+        self.library._add_lun_to_table = mock.Mock()
+        msg = 'Device busy'
+        self.mock_object(self.library.zapi_client,
+                         'clone_lun',
+                         mock.Mock(side_effect=netapp_api.NaApiError(
+                             message=msg)))
+        self.mock_object(self.library,
+                         '_retry_clone_lun',
+                         mock.Mock(return_value=None)
+                         )
+        self.library._clone_lun('fakeLUN', 'newFakeLUN', is_snapshot=True)
+        self.library.zapi_client.clone_lun.assert_called_once_with(
+            'fakeLUN', 'fakeLUN', 'newFakeLUN', 'true', block_count=0,
+            dest_block=0, src_block=0, qos_policy_group_name=None,
+            qos_policy_group_is_adaptive=False,
+            source_snapshot=None, is_snapshot=True)
+
+    def test__retry_clone_lun_success(self):
+        self.library.zapi_client = mock.Mock()
+        self.library._retry_clone_lun('fakeSourceLUN',
+                                      'fakeLUN',
+                                      'newFakeLUN',
+                                      'false',
+                                      )
+        self.library.zapi_client.clone_lun.assert_called_once_with(
+            'fakeSourceLUN', 'fakeLUN', 'newFakeLUN', 'false', block_count=0,
+            dest_block=0, src_block=0, qos_policy_group_name=None,
+            qos_policy_group_is_adaptive=False, source_snapshot=None,
+            is_snapshot=False)
+
+    def test_retry_clone_lun_failure(self):
+        self.library.zapi_client = mock.Mock()
+        self.mock_object(self.library.zapi_client,
+                         'clone_lun',
+                         mock.Mock(
+                             side_effect=na_utils.NetAppDriverException),
+                         )
+        self.assertRaises(
+            na_utils.NetAppDriverException,
+            self.library._retry_clone_lun,
+            'fakeLUN', 'fakeLUN',
+            'newFakeLUN', 'false',
+        )
+
     def test_get_fc_target_wwpns(self):
         ports = [fake.FC_FORMATTED_TARGET_WWPNS[0],
                  fake.FC_FORMATTED_TARGET_WWPNS[1]]
@@ -454,6 +550,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             'replication_enabled': False,
             'online_extend_support': True,
             'netapp_is_flexgroup': 'false',
+            'total_volumes': 2,
         }]
         if report_provisioned_capacity:
             expected[0].update({'provisioned_capacity_gb': 5.0})
@@ -548,8 +645,22 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.assertEqual({}, result)
         mock_list_flexvols.assert_called_once_with()
 
-    def test_update_ssc(self):
+    def test_update_ssc_disaggregated_platform(self):
+        """Test _update_ssc with disaggregated platform (ASA r2)."""
+        self.library.configuration.netapp_disaggregated_platform = True
+        mock_get_cluster_pool_map = self.mock_object(
+            self.library, '_get_cluster_to_pool_map',
+            return_value=fake.FAKE_CLUSTER_INFO)
 
+        result = self.library._update_ssc()
+
+        self.assertIsNone(result)
+        mock_get_cluster_pool_map.assert_called_once_with()
+        self.library.ssc_library.update_ssc_asa.assert_called_once_with(
+            fake.FAKE_CLUSTER_INFO)
+
+    def test_update_ssc(self):
+        """Test _update_ssc with traditional platform (flexvol)."""
         mock_get_pool_map = self.mock_object(
             self.library, '_get_flexvol_to_pool_map',
             return_value=fake.FAKE_CMODE_VOLUMES)
@@ -783,6 +894,70 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.assertEqual('dev1', actual_active)
         self.assertEqual([], vol_updates)
 
+    @ddt.data({'secondary_id': 'dev0', 'configured_targets': ['dev1']},
+              {'secondary_id': 'dev3', 'configured_targets': ['dev1', 'dev2']},
+              {'secondary_id': 'dev1', 'configured_targets': []},
+              {'secondary_id': None, 'configured_targets': []})
+    @ddt.unpack
+    def test_failover_invalid_replication_target(self, secondary_id,
+                                                 configured_targets):
+        """This tests executes a method in the DataMotionMixin."""
+        self.library.backend_name = 'dev0'
+        self.mock_object(data_motion.DataMotionMixin,
+                         'get_replication_backend_names',
+                         return_value=configured_targets)
+        complete_failover_call = self.mock_object(
+            data_motion.DataMotionMixin, '_complete_failover')
+
+        self.assertRaises(exception.InvalidReplicationTarget,
+                          self.library.failover, 'fake_context', [],
+                          secondary_id=secondary_id)
+        self.assertFalse(complete_failover_call.called)
+
+    def test_failover_unable_to_failover(self):
+        """This tests executes a method in the DataMotionMixin."""
+        self.library.backend_name = 'dev0'
+        self.mock_object(data_motion.DataMotionMixin, '_complete_failover',
+                         side_effect=na_utils.NetAppDriverException)
+        self.mock_object(data_motion.DataMotionMixin,
+                         'get_replication_backend_names',
+                         return_value=['dev1', 'dev2'])
+        self.mock_object(self.library.ssc_library, 'get_ssc_flexvol_names',
+                         return_value=fake_utils.SSC.keys())
+        self.mock_object(self.library, '_update_zapi_client')
+
+        self.assertRaises(exception.UnableToFailOver,
+                          self.library.failover, 'fake_context', [],
+                          secondary_id='dev1')
+        data_motion.DataMotionMixin._complete_failover.assert_called_once_with(
+            'dev0', ['dev1', 'dev2'], fake_utils.SSC.keys(), [],
+            failover_target='dev1')
+        self.assertFalse(self.library._update_zapi_client.called)
+
+    def test_failover(self):
+        """This tests executes a method in the DataMotionMixin."""
+        self.library.backend_name = 'dev0'
+        self.mock_object(data_motion.DataMotionMixin, '_complete_failover',
+                         return_value=('dev1', []))
+        self.mock_object(data_motion.DataMotionMixin,
+                         'get_replication_backend_names',
+                         return_value=['dev1', 'dev2'])
+        self.mock_object(self.library.ssc_library, 'get_ssc_flexvol_names',
+                         return_value=fake_utils.SSC.keys())
+        self.mock_object(self.library, '_update_zapi_client')
+
+        actual_active, vol_updates, __ = self.library.failover(
+            'fake_context', [], secondary_id='dev1', groups=[])
+
+        data_motion.DataMotionMixin._complete_failover.assert_called_once_with(
+            'dev0', ['dev1', 'dev2'], fake_utils.SSC.keys(), [],
+            failover_target='dev1')
+
+    def test_failover_completed(self):
+        self.mock_object(self.library, '_update_zapi_client')
+        self.library.failover_completed('fake_context', secondary_id='dev1')
+        self.library._update_zapi_client.assert_called_once_with('dev1')
+
     def test_add_looping_tasks(self):
         mock_update_ssc = self.mock_object(self.library, '_update_ssc')
         mock_handle_housekeeping = self.mock_object(
