@@ -14,11 +14,14 @@
 #    under the License.
 """Unit tests for INFINIDAT InfiniBox volume driver."""
 
+import collections
 import copy
 import functools
+import itertools
 import platform
 import socket
 from unittest import mock
+import uuid
 
 import ddt
 from oslo_utils import units
@@ -52,22 +55,31 @@ TEST_TARGET_PORTAL4 = '{}:{}'.format(TEST_IP_ADDRESS4, TEST_ISCSI_TCP_PORT2)
 TEST_FC_PROTOCOL = 'fc'
 TEST_ISCSI_PROTOCOL = 'iscsi'
 TEST_VOLUME_SOURCE_NAME = 'test-volume'
+TEST_VOLUME_TYPE = 'MASTER'
 TEST_VOLUME_SOURCE_ID = 12345
 TEST_VOLUME_METADATA = {'cinder_id': fake.VOLUME_ID}
 TEST_SNAPSHOT_SOURCE_NAME = 'test-snapshot'
 TEST_SNAPSHOT_SOURCE_ID = 67890
 TEST_SNAPSHOT_METADATA = {'cinder_id': fake.SNAPSHOT_ID}
+TEST_POOL_NAME = 'pool'
+TEST_POOL_NAME2 = 'pool2'
+TEST_SYSTEM_SERIAL = 123
+TEST_SYSTEM_SERIAL2 = 456
 
 test_volume = mock.Mock(id=fake.VOLUME_ID, name_id=fake.VOLUME_ID, size=1,
-                        volume_type_id=fake.VOLUME_TYPE_ID,
+                        volume_type_id=fake.VOLUME_TYPE_ID, group_id=None,
                         multiattach=False, volume_attachment=None)
 test_volume2 = mock.Mock(id=fake.VOLUME2_ID, name_id=fake.VOLUME2_ID, size=1,
-                         volume_type_id=fake.VOLUME_TYPE_ID,
+                         volume_type_id=None, group_id=None,
                          multiattach=False, volume_attachment=None)
-test_snapshot = mock.Mock(id=fake.SNAPSHOT_ID, volume=test_volume,
-                          volume_id=test_volume.id)
-test_clone = mock.Mock(id=fake.VOLUME4_ID, size=1, multiattach=False,
-                       volume_attachment=None)
+test_volume3 = mock.Mock(id=fake.VOLUME3_ID, name_id=fake.VOLUME3_ID, size=1,
+                         volume_type_id=fake.VOLUME_TYPE_ID,
+                         group_id=fake.GROUP_ID, multiattach=True,
+                         volume_attachment=None)
+test_snapshot = mock.Mock(id=fake.SNAPSHOT_ID, volume=test_volume)
+test_clone = mock.Mock(id=fake.VOLUME4_ID, name_id=fake.VOLUME4_ID, size=1,
+                       volume_type_id=fake.VOLUME_TYPE_ID, group_id=None,
+                       multiattach=False, volume_attachment=None)
 test_group = mock.Mock(id=fake.GROUP_ID)
 test_snapgroup = mock.Mock(id=fake.GROUP_SNAPSHOT_ID, group=test_group)
 test_connector = dict(wwpns=[TEST_WWN_1],
@@ -109,7 +121,7 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
                              configuration.SHARED_CONF_GROUP)
         self.override_config('san_password', 'password',
                              configuration.SHARED_CONF_GROUP)
-        self.override_config('infinidat_pool_name', 'pool')
+        self.override_config('infinidat_pool_name', TEST_POOL_NAME)
         self.driver = infinidat.InfiniboxVolumeDriver(
             configuration=self.configuration)
         self._system = self._infinibox_mock()
@@ -135,9 +147,8 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
         self._mock_new_volume = mock.Mock()
         self._mock_volume.get_id.return_value = TEST_VOLUME_SOURCE_ID
         self._mock_volume.get_name.return_value = TEST_VOLUME_SOURCE_NAME
-        self._mock_volume.get_type.return_value = 'MASTER'
-        self._mock_volume.get_pool_name.return_value = (
-            self.configuration.infinidat_pool_name)
+        self._mock_volume.get_type.return_value = TEST_VOLUME_TYPE
+        self._mock_volume.get_pool_name.return_value = TEST_POOL_NAME
         self._mock_volume.get_size.return_value = 1 * units.Gi
         self._mock_volume.has_children.return_value = False
         self._mock_volume.get_qos_policy.return_value = None
@@ -145,6 +156,7 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
         self._mock_volume.get_all_metadata.return_value = {}
         self._mock_volume.create_snapshot.return_value = self._mock_volume
         self._mock_snapshot = mock.Mock()
+        self._mock_snapshot.get_parent.return_value = self._mock_volume
         self._mock_host = mock.Mock()
         self._mock_host.get_luns.return_value = []
         self._mock_host.map_volume().get_lun.return_value = TEST_LUN
@@ -175,6 +187,7 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
         result.components.nodes.get_all.return_value = []
         result.qos_policies.create.return_value = self._mock_qos_policy
         result.qos_policies.safe_get.return_value = None
+        result.get_serial.return_value = TEST_SYSTEM_SERIAL
         return result
 
     def _raise_infinisdk(self, *args, **kwargs):
@@ -205,6 +218,14 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         self._mock_host.set_metadata_from_dict.assert_called_once_with(
             self._generate_mock_host_metadata())
 
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_get_oslo_driver_opts')
+    def test_get_driver_options(self, _get_oslo_driver_opts):
+        _get_oslo_driver_opts.return_value = []
+        result = self.driver.get_driver_options()
+        actual = (infinidat.infinidat_opts)
+        self.assertEqual(actual, result)
+
     @skip_driver_setup
     def test__setup_and_get_system_object(self):
         # This test should skip the driver setup, as it generates more calls to
@@ -220,6 +241,12 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
             infinidat._INFINIDAT_CINDER_IDENTIFIER)
         self._system.login.assert_called_once()
 
+    @skip_driver_setup
+    @mock.patch('cinder.volume.drivers.infinidat.infinisdk', None)
+    def test_do_setup_no_infinisdk(self):
+        self.assertRaises(exception.VolumeDriverException,
+                          self.driver.do_setup, None)
+
     @mock.patch('cinder.volume.drivers.infinidat.infinisdk.InfiniBox')
     @ddt.data(True, False)
     def test_ssl_options(self, use_ssl, infinibox):
@@ -230,9 +257,31 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         infinibox.assert_called_once_with(self.configuration.san_ip,
                                           auth=auth, use_ssl=use_ssl)
 
-    def test_initialize_connection(self):
+    def test_create_export_snapshot(self):
+        self.assertIsNone(self.driver.create_export_snapshot(
+            None, test_snapshot, test_connector))
+
+    def test_remove_export_snapshot(self):
+        self.assertIsNone(self.driver.remove_export_snapshot(
+            None, test_snapshot))
+
+    def test_backup_use_temp_snapshot(self):
+        self.assertTrue(self.driver.backup_use_temp_snapshot())
+
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_get_infinidat_snapshot')
+    def test_initialize_connection_snapshot(self, get_snapshot):
+        result = self.driver.initialize_connection_snapshot(
+            test_snapshot, test_connector)
+        get_snapshot.assert_called_once_with(test_snapshot)
+        self.assertEqual(1, result["data"]["target_lun"])
+
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_get_infinidat_volume')
+    def test_initialize_connection(self, get_volume):
         self._system.hosts.safe_get.return_value = None
         result = self.driver.initialize_connection(test_volume, test_connector)
+        get_volume.assert_called_once_with(test_volume)
         self.assertEqual(1, result["data"]["target_lun"])
 
     def test_initialize_connection_host_exists(self):
@@ -247,6 +296,13 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         result = self.driver.initialize_connection(test_volume, test_connector)
         self.assertEqual(888, result["data"]["target_lun"])
 
+    def test_initialize_connection_mapping_not_found(self):
+        mock_mapping = mock.Mock()
+        mock_mapping.get_volume.return_value = None
+        self._mock_host.get_luns.return_value = [mock_mapping]
+        result = self.driver.initialize_connection(test_volume, test_connector)
+        self.assertEqual(TEST_LUN, result["data"]["target_lun"])
+
     def test_initialize_connection_volume_doesnt_exist(self):
         self._system.volumes.safe_get.return_value = None
         self.assertRaises(exception.VolumeNotFound,
@@ -290,11 +346,37 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         self.assertFalse(self.driver._is_volume_multiattached(volume,
                                                               connector))
 
-    def test_terminate_connection(self):
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_terminate_connection')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_get_infinidat_volume')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_is_volume_multiattached')
+    def test_terminate_connection(self, volume_multiattached, get_volume,
+                                  terminate_connection):
         volume = copy.deepcopy(test_volume)
         volume.volume_attachment = [test_attachment1]
+        volume_multiattached.return_value = False
+        get_volume.return_value = self._mock_volume
         self.assertFalse(self.driver.terminate_connection(volume,
                                                           test_connector))
+        volume_multiattached.assert_called_once_with(volume, test_connector)
+        get_volume.assert_called_once_with(volume)
+        terminate_connection.assert_called_once_with(self._mock_volume,
+                                                     test_connector)
+
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_terminate_connection')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_get_infinidat_snapshot')
+    def test_terminate_connection_snapshot(self, get_snapshot,
+                                           terminate_connection):
+        get_snapshot.return_value = self._mock_snapshot
+        self.assertIsNone(self.driver.terminate_connection_snapshot(
+            test_snapshot, test_connector))
+        get_snapshot.assert_called_once_with(test_snapshot)
+        terminate_connection.assert_called_once_with(self._mock_snapshot,
+                                                     test_connector)
 
     def test_terminate_connection_delete_host(self):
         self._mock_host.get_luns.return_value = [object()]
@@ -373,7 +455,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         )
 
     @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
-    def test_create_volume_compression_not_enabled(self, *mocks):
+    def test_create_volume_compression_disabled(self, *mocks):
         self.override_config('infinidat_use_compression', False)
         self.driver.create_volume(test_volume)
         self.assertFalse(
@@ -381,14 +463,29 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         )
 
     @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
-    def test_create_volume_compression_not_available(self, *mocks):
-        self._system.compat.has_compression.return_value = False
+    def test_create_volume_compression_default(self, *mocks):
         self.driver.create_volume(test_volume)
         self.assertNotIn(
             "compression_enabled",
             self._system.volumes.create.call_args[1]
         )
 
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.volume_utils.group_get_by_id')
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=True)
+    def test_create_volume_within_group(self, *mocks):
+        self.driver.create_volume(test_volume3)
+        self._mock_group.add_member.assert_called_once()
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.volume_utils.group_get_by_id')
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_create_volume_within_no_cg_group(self, *mocks):
+        self.driver.create_volume(test_volume3)
+        self._mock_group.add_member.assert_not_called()
+
     def test_delete_volume(self):
         self.driver.delete_volume(test_volume)
 
@@ -444,7 +541,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                           test_clone, test_snapshot)
 
     def test_create_volume_from_snapshot_create_fails(self):
-        self._mock_volume.create_snapshot.side_effect = self._raise_infinisdk
+        self._system.volumes.create.side_effect = self._raise_infinisdk
         self.assertRaises(exception.VolumeBackendAPIException,
                           self.driver.create_volume_from_snapshot,
                           test_clone, test_snapshot)
@@ -458,13 +555,18 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                           self.driver.create_volume_from_snapshot,
                           test_clone, test_snapshot)
 
-    @mock.patch("cinder.volume.volume_utils.copy_volume")
-    @mock.patch("cinder.volume.volume_utils.brick_get_connector")
-    @mock.patch("cinder.volume.volume_utils.brick_get_connector_properties",
-                return_value=test_connector)
-    def test_create_volume_from_snapshot_delete_clone_fails(self, *mocks):
-        self._mock_volume.delete.side_effect = self._raise_infinisdk
-        self.assertRaises(exception.VolumeBackendAPIException,
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector')
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector_properties')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_connect_device')
+    def test_create_volume_from_snapshot_connect_fails(self, connect_device,
+                                                       connector_properties,
+                                                       *mocks):
+        connector_properties.return_value = test_connector
+        connect_device.side_effect = exception.DeviceUnavailable(
+            path='/dev/sdb', reason='Block device required')
+        self.assertRaises(exception.DeviceUnavailable,
                           self.driver.create_volume_from_snapshot,
                           test_clone, test_snapshot)
 
@@ -481,21 +583,28 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         self.assertRaises(exception.VolumeBackendAPIException,
                           self.driver.delete_snapshot, test_snapshot)
 
-    @mock.patch("cinder.volume.volume_utils.copy_volume")
-    @mock.patch("cinder.volume.volume_utils.brick_get_connector")
-    @mock.patch("cinder.volume.volume_utils.brick_get_connector_properties",
-                return_value=test_connector)
-    @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
-    def test_create_cloned_volume(self, *mocks):
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                'delete_snapshot')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                'create_volume_from_snapshot')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                'create_snapshot')
+    @mock.patch('uuid.uuid4')
+    def test_create_cloned_volume(self, mock_uuid, create_snapshot,
+                                  create_volume_from_snapshot,
+                                  delete_snapshot):
+        mock_uuid.return_value = uuid.UUID(test_snapshot.id)
+        snapshot_attributes = ('id', 'name', 'volume')
+        Snapshot = collections.namedtuple('Snapshot', snapshot_attributes)
+        snapshot_id = test_snapshot.id
+        snapshot_name = self.configuration.snapshot_name_template % snapshot_id
+        snapshot = Snapshot(id=snapshot_id, name=snapshot_name,
+                            volume=test_volume)
         self.driver.create_cloned_volume(test_clone, test_volume)
-
-    def test_create_cloned_volume_volume_already_mapped(self):
-        mock_mapping = mock.Mock()
-        mock_mapping.get_volume.return_value = self._mock_volume
-        self._mock_volume.get_logical_units.return_value = [mock_mapping]
-        self.assertRaises(exception.VolumeBackendAPIException,
-                          self.driver.create_cloned_volume,
-                          test_clone, test_volume)
+        create_snapshot.assert_called_once_with(snapshot)
+        create_volume_from_snapshot.assert_called_once_with(test_clone,
+                                                            snapshot)
+        delete_snapshot.assert_called_once_with(snapshot)
 
     def test_create_cloned_volume_create_fails(self):
         self._system.volumes.create.side_effect = self._raise_infinisdk
@@ -517,6 +626,13 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
     def test_create_group(self, *mocks):
         self.driver.create_group(None, test_group)
 
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_create_generic_group(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.create_group,
+                          None, test_group)
+
     @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
                 return_value=True)
     def test_create_group_metadata(self, *mocks):
@@ -542,6 +658,13 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
     def test_delete_group(self, *mocks):
         self.driver.delete_group(None, test_group, [test_volume])
 
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_delete_generic_group(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.delete_group,
+                          None, test_group, [test_volume])
+
     @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
                 return_value=True)
     def test_delete_group_doesnt_exist(self, *mocks):
@@ -562,6 +685,13 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         self.driver.update_group(None, test_group,
                                  [test_volume], [test_volume])
 
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_update_generic_group_add_and_remove(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.update_group,
+                          None, test_group, [test_volume], [test_volume])
+
     @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
                 return_value=True)
     def test_update_group_api_fail(self, *mocks):
@@ -583,6 +713,23 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                                           test_snapgroup, [test_snapshot],
                                           None, None)
 
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_create_genericgroup_from_src_snaps(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.create_group_from_src,
+                          None, test_group, [test_volume],
+                          test_snapgroup, [test_snapshot],
+                          None, None)
+
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=True)
+    def test_create_group_from_empty_sources(self, *mocks):
+        self.assertRaises(exception.InvalidInput,
+                          self.driver.create_group_from_src,
+                          None, test_group, [test_volume],
+                          None, None, None, None)
+
     @mock.patch("cinder.volume.volume_utils.copy_volume")
     @mock.patch("cinder.volume.volume_utils.brick_get_connector")
     @mock.patch("cinder.volume.volume_utils.brick_get_connector_properties",
@@ -599,13 +746,20 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                 return_value=True)
     def test_create_group_snap(self, *mocks):
         mock_snapgroup = mock.Mock()
-        mock_snapgroup.get_members.return_value = [self._mock_volume]
-        self._mock_volume.get_parent.return_value = self._mock_volume
-        self._mock_volume.get_name.return_value = ''
+        mock_snapgroup.get_members.return_value = [self._mock_snapshot,
+                                                   self._mock_snapshot]
+        self._mock_volume.get_name.side_effect = [fake.VOLUME_NAME,
+                                                  fake.VOLUME2_NAME]
         self._mock_group.create_snapshot.return_value = mock_snapgroup
-        self.driver.create_group_snapshot(None,
-                                          test_snapgroup,
-                                          [test_snapshot])
+        self.driver.create_group_snapshot(None, test_snapgroup,
+                                          [test_snapshot, test_snapshot])
+
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_create_generic_group_snap(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.create_group_snapshot,
+                          None, test_snapgroup, [test_snapshot])
 
     @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
                 return_value=True)
@@ -622,6 +776,13 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                                           test_snapgroup,
                                           [test_snapshot])
 
+    @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
+                return_value=False)
+    def test_delete_generic_group_snap(self, *mocks):
+        self.assertRaises(NotImplementedError,
+                          self.driver.delete_group_snapshot,
+                          None, test_snapgroup, [test_snapshot])
+
     @mock.patch('cinder.volume.volume_utils.is_group_a_cg_snapshot_type',
                 return_value=True)
     def test_delete_group_snap_does_not_exist(self, *mocks):
@@ -646,21 +807,6 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                           self.driver.delete_group_snapshot,
                           None, test_snapgroup, [test_snapshot])
 
-    def test_terminate_connection_force_detach(self):
-        mock_infinidat_host = mock.Mock()
-        mock_infinidat_host.get_ports.return_value = [
-            self._wwn.WWN(TEST_WWN_1)]
-        mock_mapping = mock.Mock()
-        mock_mapping.get_host.return_value = mock_infinidat_host
-        self._mock_volume.get_logical_units.return_value = [mock_mapping]
-        volume = copy.deepcopy(test_volume)
-        volume.volume_attachment = [test_attachment1, test_attachment2]
-        # connector is None - force detach - detach all mappings
-        self.assertTrue(self.driver.terminate_connection(volume, None))
-        # make sure we actually detached the host mapping
-        self._mock_host.unmap_volume.assert_called_once()
-        self._mock_host.safe_delete.assert_called_once()
-
     def test_snapshot_revert_use_temp_snapshot(self):
         result = self.driver.snapshot_revert_use_temp_snapshot()
         self.assertFalse(result)
@@ -724,7 +870,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
     @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
     def test_manage_existing_invalid_pool(self, *mocks):
         existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
-        self._mock_volume.get_pool_name.return_value = 'invalid'
+        self._mock_volume.get_pool_name.return_value = TEST_POOL_NAME2
         self.assertRaises(exception.InvalidConfigurationValue,
                           self.driver.manage_existing,
                           test_volume, existing_ref)
@@ -840,7 +986,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
     @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
     def test_manage_existing_snapshot_invalid_pool(self, *mocks):
         existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
-        self._mock_volume.get_pool_name.return_value = 'invalid'
+        self._mock_volume.get_pool_name.return_value = TEST_POOL_NAME2
         self.assertRaises(exception.InvalidConfigurationValue,
                           self.driver.manage_existing_snapshot,
                           test_snapshot, existing_ref)
@@ -976,9 +1122,87 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                                              test_volume)
         self.assertEqual(0, self._log.error.call_count)
 
+    @ddt.data(None, {})
+    def test_migrate_volume_no_host(self, host):
+        expected = False, None
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    @ddt.data(None, {})
+    def test_migrate_volume_no_capabilities(self, capabilities):
+        expected = False, None
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    @ddt.data(None, 123, 'location')
+    def test_migrate_volume_invalid_location_info(self, location_info):
+        expected = False, None
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    def test_migrate_volume_invalid_driver(self):
+        expected = False, None
+        location_info = 'vendor:0:/path'
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    def test_migrate_volume_invalid_serial(self):
+        expected = False, None
+        location_info = '%s:%s:%s' % (self.driver.__class__.__name__,
+                                      TEST_SYSTEM_SERIAL2, TEST_POOL_NAME2)
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    def test_migrate_volume_same_pool(self):
+        expected = True, None
+        location_info = '%s:%s:%s' % (self.driver.__class__.__name__,
+                                      TEST_SYSTEM_SERIAL, TEST_POOL_NAME)
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    def test_migrate_volume_no_pool(self):
+        expected = False, None
+        self._system.pools.safe_get.return_value = None
+        location_info = '%s:%s:%s' % (self.driver.__class__.__name__,
+                                      TEST_SYSTEM_SERIAL, TEST_POOL_NAME2)
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
+    def test_migrate_volume(self):
+        expected = True, None
+        location_info = '%s:%s:%s' % (self.driver.__class__.__name__,
+                                      TEST_SYSTEM_SERIAL, TEST_POOL_NAME2)
+        capabilities = {'location_info': location_info}
+        host = {'capabilities': capabilities}
+        update = self.driver.migrate_volume(None, test_volume, host)
+        self.assertEqual(expected, update)
+
 
 @ddt.ddt
 class InfiniboxDriverTestCaseFC(InfiniboxDriverTestCaseBase):
+    @ddt.data(*itertools.product(('UP', 'DOWN'), ('OK', 'ERROR')))
+    @ddt.unpack
+    def test_initialize_connection_nodes_ports(self, link_state, port_state):
+        node = mock.Mock()
+        port = mock.Mock()
+        port.get_link_state.return_value = link_state
+        port.get_state.return_value = port_state
+        node.get_fc_ports.return_value = [port]
+        self._system.components.nodes.get_all.return_value = [node]
+        result = self.driver.initialize_connection(test_volume, test_connector)
+        self.assertEqual(1, result["data"]["target_lun"])
+
     def test_initialize_connection_multiple_wwpns(self):
         connector = {'wwpns': [TEST_WWN_1, TEST_WWN_2]}
         result = self.driver.initialize_connection(test_volume, connector)
@@ -1010,6 +1234,19 @@ class InfiniboxDriverTestCaseFC(InfiniboxDriverTestCaseBase):
         self.assertTrue(self.driver.terminate_connection(volume,
                                                          test_connector))
 
+    def test_terminate_connection_force_detach(self):
+        mock_infinidat_host = mock.Mock()
+        mock_infinidat_host.get_ports.return_value = [
+            self._wwn.WWN(TEST_WWN_1)]
+        mock_mapping = mock.Mock()
+        mock_mapping.get_host.return_value = mock_infinidat_host
+        self._mock_volume.get_logical_units.return_value = [mock_mapping]
+        volume = copy.deepcopy(test_volume)
+        volume.volume_attachment = [test_attachment1, test_attachment2]
+        self.assertTrue(self.driver.terminate_connection(volume, None))
+        self._mock_host.unmap_volume.assert_called_once()
+        self._mock_host.safe_delete.assert_called_once()
+
 
 @ddt.ddt
 class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
@@ -1212,6 +1449,19 @@ class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
         self.assertFalse(self.driver.terminate_connection(volume,
                                                           test_connector))
 
+    def test_terminate_connection_force_detach(self):
+        mock_infinidat_host = mock.Mock()
+        mock_infinidat_host.get_ports.return_value = [
+            self._iqn.IQN(TEST_TARGET_IQN)]
+        mock_mapping = mock.Mock()
+        mock_mapping.get_host.return_value = mock_infinidat_host
+        self._mock_volume.get_logical_units.return_value = [mock_mapping]
+        volume = copy.deepcopy(test_volume)
+        volume.volume_attachment = [test_attachment1, test_attachment2]
+        self.assertTrue(self.driver.terminate_connection(volume, None))
+        self._mock_host.unmap_volume.assert_called_once()
+        self._mock_host.safe_delete.assert_called_once()
+
     def test_validate_connector(self):
         fc_connector = {'wwpns': [TEST_WWN_1, TEST_WWN_2]}
         iscsi_connector = {'initiator': TEST_INITIATOR_IQN}
@@ -1221,6 +1471,13 @@ class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
 
 
 class InfiniboxDriverTestCaseQoS(InfiniboxDriverTestCaseBase):
+    @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
+    def test_no_qos(self, qos_specs):
+        qos_specs.return_value = None
+        self.driver.create_volume(test_volume)
+        self._system.qos_policies.create.assert_not_called()
+        self._mock_qos_policy.assign_entity.assert_not_called()
+
     @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
     def test_qos_max_ipos(self, qos_specs):
         qos_specs.return_value = {'qos_specs': {'id': 'qos_name',
@@ -1258,8 +1515,7 @@ class InfiniboxDriverTestCaseQoS(InfiniboxDriverTestCaseBase):
                                                 'consumer': 'back-end',
                                                 'specs': {'maxIOPS': 1000,
                                                           'maxBWS': 10000}}}
-        test_volume = mock.Mock(id=1, size=1, volume_type_id=None)
-        self.driver.create_volume(test_volume)
+        self.driver.create_volume(test_volume2)
         self._system.qos_policies.create.assert_not_called()
         self._mock_qos_policy.assign_entity.assert_not_called()
 
