@@ -17,6 +17,7 @@ from unittest import mock
 import ddt
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils import importutils
 
 from cinder.brick.local_dev import lvm as brick_lvm
 from cinder import db
@@ -47,6 +48,71 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
     FAKE_VOLUME = {'name': 'test1',
                    'id': 'test1'}
 
+    def test___init___share_target_not_supported(self):
+        """Fail to use shared targets if target driver doesn't support it."""
+        original_import = importutils.import_object
+
+        def wrap_target_as_no_shared_support(*args, **kwargs):
+            res = original_import(*args, **kwargs)
+            self.mock_object(res, 'SHARED_TARGET_SUPPORT', False)
+            return res
+
+        self.patch('oslo_utils.importutils.import_object',
+                   side_effect=wrap_target_as_no_shared_support)
+
+        self.configuration.lvm_share_target = True
+        self.assertRaises(exception.InvalidConfigurationValue,
+                          lvm.LVMVolumeDriver,
+                          configuration=self.configuration)
+
+    def test___init___secondary_ips_not_supported(self):
+        """Fail to use secondary ips if target driver doesn't support it."""
+        original_import = importutils.import_object
+
+        def wrap_target_as_no_secondary_ips_support(*args, **kwargs):
+            res = original_import(*args, **kwargs)
+            self.mock_object(res, 'SECONDARY_IP_SUPPORT', False)
+            return res
+
+        self.patch('oslo_utils.importutils.import_object',
+                   side_effect=wrap_target_as_no_secondary_ips_support)
+
+        self.configuration.target_secondary_ip_addresses = True
+        self.assertRaises(exception.InvalidConfigurationValue,
+                          lvm.LVMVolumeDriver,
+                          configuration=self.configuration)
+
+    def test___init___share_target_supported(self):
+        """OK to use shared targets if target driver supports it."""
+        original_import = importutils.import_object
+
+        def wrap_target_as_no_shared_support(*args, **kwargs):
+            res = original_import(*args, **kwargs)
+            self.mock_object(res, 'SHARED_TARGET_SUPPORT', True)
+            return res
+
+        self.patch('oslo_utils.importutils.import_object',
+                   side_effect=wrap_target_as_no_shared_support)
+
+        self.configuration.lvm_share_target = True
+        lvm.LVMVolumeDriver(configuration=self.configuration)
+
+    @ddt.data(True, False)
+    def test___init___share_target_not_requested(self, supports_shared):
+        """For non shared it works regardless of target driver support."""
+        original_import = importutils.import_object
+
+        def wrap_target_as_no_shared_support(*args, **kwargs):
+            res = original_import(*args, **kwargs)
+            self.mock_object(res, 'SHARED_TARGET_SUPPORT', supports_shared)
+            return res
+
+        self.patch('oslo_utils.importutils.import_object',
+                   side_effect=wrap_target_as_no_shared_support)
+
+        self.configuration.lvm_share_target = False
+        lvm.LVMVolumeDriver(configuration=self.configuration)
+
     @mock.patch.object(os.path, 'exists', return_value=True)
     @mock.patch.object(fake_driver.FakeLoggingVolumeDriver, 'create_export')
     def test_delete_volume_invalid_parameter(self, _mock_create_export,
@@ -718,20 +784,20 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
             self.context, fake_volume.id)
 
         with mock.patch.object(self.volume.driver.vg,
-                               'revert') as mock_revert,\
+                               'revert') as mock_revert, \
                 mock.patch.object(self.volume.driver.vg,
-                                  'create_lv_snapshot') as mock_create,\
+                                  'create_lv_snapshot') as mock_create, \
                 mock.patch.object(self.volume.driver.vg,
-                                  'deactivate_lv') as mock_deactive,\
+                                  'deactivate_lv') as mock_deactivate, \
                 mock.patch.object(self.volume.driver.vg,
-                                  'activate_lv') as mock_active:
+                                  'activate_lv') as mock_activate:
             self.volume.driver.revert_to_snapshot(self.context,
                                                   fake_volume,
                                                   fake_snapshot)
             mock_revert.assert_called_once_with(
                 self.volume.driver._escape_snapshot(fake_snapshot.name))
-            mock_deactive.assert_called_once_with(fake_volume.name)
-            mock_active.assert_called_once_with(fake_volume.name)
+            mock_deactivate.assert_called_once_with(fake_volume.name)
+            mock_activate.assert_called_once_with(fake_volume.name)
             mock_create.assert_called_once_with(
                 self.volume.driver._escape_snapshot(fake_snapshot.name),
                 fake_volume.name, self.configuration.lvm_type)
@@ -1029,6 +1095,10 @@ class LVMISCSITestCase(test_driver.BaseDriverTestCase):
         lvm_driver = lvm.LVMVolumeDriver(
             configuration=self.configuration, vg_obj=vg_obj)
 
+        mock_same = self.mock_object(
+            lvm_driver.target_driver, 'are_same_connector',
+            side_effect=lvm_driver.target_driver.are_same_connector)
+
         with mock.patch.object(lvm_driver.target_driver,
                                'terminate_connection') as mock_term_conn:
 
@@ -1037,14 +1107,31 @@ class LVMISCSITestCase(test_driver.BaseDriverTestCase):
             self.assertTrue(lvm_driver.terminate_connection(vol,
                                                             host1_connector))
             mock_term_conn.assert_not_called()
+            self.assertEqual(3, mock_same.call_count)
+            mock_same.assert_has_calls((
+                mock.call(host1_connector, host1_connector),
+                mock.call(host1_connector, host1_connector),
+                mock.call(host2_connector, host1_connector)))
+            mock_same.reset_mock()
 
             # Verify that terminate_connection is called against either host
             # when only one active attachment per host is present.
             vol.volume_attachment.objects.remove(host1_attachment1)
             self.assertTrue(lvm_driver.terminate_connection(vol,
                                                             host1_connector))
+            self.assertEqual(2, mock_same.call_count)
+            mock_same.assert_has_calls((
+                mock.call(host1_connector, host1_connector),
+                mock.call(host2_connector, host1_connector)))
+            mock_same.reset_mock()
+
             self.assertTrue(lvm_driver.terminate_connection(vol,
                                                             host2_connector))
+            self.assertEqual(2, mock_same.call_count)
+            mock_same.assert_has_calls((
+                mock.call(host1_connector, host2_connector),
+                mock.call(host2_connector, host2_connector)))
+            mock_same.reset_mock()
             mock_term_conn.assert_has_calls([mock.call(vol, host1_connector),
                                              mock.call(vol, host2_connector)])
 
