@@ -50,13 +50,14 @@ class RemoteFsDriverTestCase(test.TestCase):
     def setUp(self):
         super(RemoteFsDriverTestCase, self).setUp()
         self._driver = remotefs.RemoteFSDriver()
-        self.configuration = mock.Mock(conf.Configuration)
-        self.configuration.append_config_values(mock.ANY)
-        self.configuration.nas_secure_file_permissions = 'false'
-        self.configuration.nas_secure_file_operations = 'false'
-        self.configuration.nfs_snapshot_support = True
-        self.configuration.max_over_subscription_ratio = 1.0
-        self.configuration.reserved_percentage = 5
+        self.configuration = conf.Configuration(None)
+        self.configuration.append_config_values(nfs.nfs_opts)
+        self.configuration.append_config_values(remotefs.nas_opts)
+        self.override_config('nas_secure_file_permissions', 'false')
+        self.override_config('nas_secure_file_operations', 'false')
+        self.override_config('nfs_snapshot_support', True)
+        self.override_config('max_over_subscription_ratio', 1.0)
+        self.override_config('reserved_percentage', 5)
         self._driver = remotefs.RemoteFSDriver(
             configuration=self.configuration)
         mock_exc = mock.patch.object(self._driver, '_execute')
@@ -92,14 +93,14 @@ class RemoteFsDriverTestCase(test.TestCase):
     @mock.patch.object(remotefs, 'LOG')
     def test_set_rw_permissions_with_secure_file_permissions(self, LOG):
         self._driver._mounted_shares = [self.TEST_EXPORT]
-        self.configuration.nas_secure_file_permissions = 'true'
+        self.override_config('nas_secure_file_permissions', 'true')
         self._driver._set_rw_permissions(self.TEST_FILE_NAME)
 
         self.assertFalse(LOG.warning.called)
 
     @mock.patch.object(remotefs, 'LOG')
     def test_set_rw_permissions_without_secure_file_permissions(self, LOG):
-        self.configuration.nas_secure_file_permissions = 'false'
+        self.override_config('nas_secure_file_permissions', 'false')
         self._driver._set_rw_permissions(self.TEST_FILE_NAME)
 
         self.assertTrue(LOG.warning.called)
@@ -290,7 +291,7 @@ class RemoteFsDriverTestCase(test.TestCase):
         operations. This test verifies the settings when secure.
         """
         drv = self._driver
-        self.configuration.nas_secure_file_operations = 'true'
+        self.override_config('nas_secure_file_operations', 'true')
         ret_flag = drv.secure_file_operations_enabled()
         self.assertTrue(ret_flag)
 
@@ -301,7 +302,7 @@ class RemoteFsDriverTestCase(test.TestCase):
         operations. This test verifies the settings when not secure.
         """
         drv = self._driver
-        self.configuration.nas_secure_file_operations = 'false'
+        self.override_config('nas_secure_file_operations', 'false')
         ret_flag = drv.secure_file_operations_enabled()
         self.assertFalse(ret_flag)
 
@@ -335,102 +336,112 @@ NFS_CONFIG4 = {'max_over_subscription_ratio': 20.0,
                'nas_secure_file_permissions': 'false',
                'nas_secure_file_operations': 'true'}
 
-QEMU_IMG_INFO_OUT1 = """image: %(volid)s
-        file format: raw
-        virtual size: %(size_gb)sG (%(size_b)s bytes)
-        disk size: 173K
-        """
+QEMU_IMG_INFO_OUT1 = """{
+    "filename": "%(volid)s",
+    "format": "raw",
+    "virtual-size": %(size_b)s,
+    "actual-size": 173000
+}"""
 
-QEMU_IMG_INFO_OUT2 = """image: %(volid)s
-file format: qcow2
-virtual size: %(size_gb)sG (%(size_b)s bytes)
-disk size: 196K
-cluster_size: 65536
-Format specific information:
-    compat: 1.1
-    lazy refcounts: false
-    refcount bits: 16
-    corrupt: false
-    """
+QEMU_IMG_INFO_OUT2 = """{
+    "filename": "%(volid)s",
+    "format": "qcow2",
+    "virtual-size": %(size_b)s,
+    "actual-size": 196000,
+    "cluster-size": 65536,
+    "format-specific": {
+        "compat": "1.1",
+        "lazy-refcounts": false,
+        "refcount-bits": 16,
+        "corrupt": false
+    }
+}"""
 
-QEMU_IMG_INFO_OUT3 = """image: volume-%(volid)s.%(snapid)s
-file format: qcow2
-virtual size: %(size_gb)sG (%(size_b)s bytes)
-disk size: 196K
-cluster_size: 65536
-backing file: volume-%(volid)s
-backing file format: qcow2
-Format specific information:
-    compat: 1.1
-    lazy refcounts: false
-    refcount bits: 16
-    corrupt: false
-    """
+QEMU_IMG_INFO_OUT3 = """{
+    "filename": "volume-%(volid)s.%(snapid)s",
+    "format": "qcow2",
+    "virtual-size": %(size_b)s,
+    "actual-size": 196000,
+    "cluster-size": 65536,
+    "backing-filename": "volume-%(volid)s",
+    "backing-filename-format": "qcow2",
+    "format-specific": {
+        "compat": "1.1",
+        "lazy-refcounts": false,
+        "refcount-bits": 16,
+        "corrupt": false
+    }
+}"""
 
-QEMU_IMG_INFO_OUT4 = """image: volume-%(volid)s.%(snapid)s
-file format: raw
-virtual size: %(size_gb)sG (%(size_b)s bytes)
-disk size: 196K
-cluster_size: 65536
-backing file: volume-%(volid)s
-backing file format: raw
-Format specific information:
-    compat: 1.1
-    lazy refcounts: false
-    refcount bits: 16
-    corrupt: false
-    """
+QEMU_IMG_INFO_OUT4 = """{
+    "filename": "volume-%(volid)s.%(snapid)s",
+    "format": "raw",
+    "virtual-size": %(size_b)s,
+    "actual-size": 196000,
+    "cluster-size": 65536,
+    "backing-filename": "volume-%(volid)s",
+    "backing-filename-format": "raw"
+}"""
 
-QEMU_IMG_INFO_OUT5 = """image: volume-%(volid)s.%(snapid)s
-file format: qcow2
-virtual size: %(size_gb)sG (%(size_b)s bytes)
-disk size: 196K
-encrypted: yes
-cluster_size: 65536
-backing file: volume-%(volid)s
-backing file format: raw
-Format specific information:
-    compat: 1.1
-    lazy refcounts: false
-    refcount bits: 16
-    encrypt:
-        ivgen alg: plain64
-        hash alg: sha256
-        cipher alg: aes-256
-        uuid: 386f8626-33f0-4683-a517-78ddfe385e33
-        format: luks
-        cipher mode: xts
-        slots:
-            [0]:
-                active: true
-                iters: 1892498
-                key offset: 4096
-                stripes: 4000
-            [1]:
-                active: false
-                key offset: 262144
-            [2]:
-                active: false
-                key offset: 520192
-            [3]:
-                active: false
-                key offset: 778240
-            [4]:
-                active: false
-                key offset: 1036288
-            [5]:
-                active: false
-                key offset: 1294336
-            [6]:
-                active: false
-                key offset: 1552384
-            [7]:
-                active: false
-                key offset: 1810432
-        payload offset: 2068480
-        master key iters: 459347
-    corrupt: false
-"""
+QEMU_IMG_INFO_OUT5 = """{
+    "filename": "volume-%(volid)s.%(snapid)s",
+    "format": "qcow2",
+    "virtual-size": %(size_b)s,
+    "actual-size": 196000,
+    "encrypted": true,
+    "cluster-size": 65536,
+    "backing-filename": "volume-%(volid)s",
+    "backing-filename-format": "raw",
+    "format-specific": {
+        "type": "luks",
+        "data": {
+            "ivgen-alg": "plain64",
+            "hash-alg": "sha256",
+            "cipher-alg": "aes-256",
+            "uuid": "386f8626-33f0-4683-a517-78ddfe385e33",
+            "cipher-mode": "xts",
+            "slots": [
+                {
+                    "active": true,
+                    "iters": 1892498,
+                    "key offset": 4096,
+                    "stripes": 4000
+                },
+                {
+                    "active": false,
+                    "key offset": 262144
+                },
+                {
+                    "active": false,
+                    "key offset": 520192
+                },
+                {
+                    "active": false,
+                    "key offset": 778240
+                },
+                {
+                    "active": false,
+                    "key offset": 1036288
+                },
+                {
+                    "active": false,
+                    "key offset": 1294336
+                },
+                {
+                    "active": false,
+                    "key offset": 1552384
+                },
+                {
+                    "active": false,
+                    "key offset": 1810432
+                }
+            ],
+            "payload-offset": 2068480,
+            "master-key-iters": 459347
+        },
+        "corrupt": false
+    }
+}"""
 
 
 @ddt.ddt
@@ -454,23 +465,24 @@ class NfsDriverTestCase(test.TestCase):
 
     def setUp(self):
         super(NfsDriverTestCase, self).setUp()
-        self.configuration = mock.Mock(conf.Configuration)
-        self.configuration.append_config_values(mock.ANY)
-        self.configuration.max_over_subscription_ratio = 1.0
-        self.configuration.reserved_percentage = 5
-        self.configuration.nfs_shares_config = None
-        self.configuration.nfs_sparsed_volumes = True
-        self.configuration.nfs_reserved_percentage = 5.0
-        self.configuration.nfs_mount_point_base = self.TEST_MNT_POINT_BASE
-        self.configuration.nfs_mount_options = None
-        self.configuration.nfs_mount_attempts = 3
-        self.configuration.nfs_qcow2_volumes = False
-        self.configuration.nas_secure_file_permissions = 'false'
-        self.configuration.nas_secure_file_operations = 'false'
-        self.configuration.nas_host = None
-        self.configuration.nas_share_path = None
-        self.configuration.nas_mount_options = None
-        self.configuration.volume_dd_blocksize = '1M'
+        self.configuration = conf.Configuration(None)
+        self.configuration.append_config_values(nfs.nfs_opts)
+        self.configuration.append_config_values(remotefs.nas_opts)
+        self.override_config('max_over_subscription_ratio', 1.0)
+        self.override_config('reserved_percentage', 5)
+        self.override_config('nfs_shares_config', None)
+        self.override_config('nfs_sparsed_volumes', True)
+        self.override_config('reserved_percentage', 5.0)
+        self.override_config('nfs_mount_point_base', self.TEST_MNT_POINT_BASE)
+        self.override_config('nfs_mount_options', None)
+        self.override_config('nfs_mount_attempts', 3)
+        self.override_config('nfs_qcow2_volumes', False)
+        self.override_config('nas_secure_file_permissions', 'false')
+        self.override_config('nas_secure_file_operations', 'false')
+        self.override_config('nas_host', None)
+        self.override_config('nas_share_path', None)
+        self.override_config('nas_mount_options', None)
+        self.override_config('volume_dd_blocksize', '1M')
 
         self.mock_object(volume_utils, 'get_max_over_subscription_ratio',
                          return_value=1)
@@ -491,7 +503,7 @@ class NfsDriverTestCase(test.TestCase):
     @ddt.data(NFS_CONFIG1, NFS_CONFIG2, NFS_CONFIG3, NFS_CONFIG4)
     def test_local_path(self, nfs_config):
         """local_path common use case."""
-        self.configuration.nfs_mount_point_base = self.TEST_MNT_POINT_BASE
+        self.override_config('nfs_mount_point_base', self.TEST_MNT_POINT_BASE)
         self._set_driver(extra_confs=nfs_config)
         drv = self._driver
 
@@ -525,7 +537,7 @@ class NfsDriverTestCase(test.TestCase):
 
         mock_fetch.assert_called_once_with(
             None, None, None, test_img_source, mock.ANY, run_as_root=True,
-            size=self.TEST_SIZE_IN_GB)
+            size=self.TEST_SIZE_IN_GB, disable_sparse=False)
         mock_resize.assert_called_once_with(test_img_source,
                                             self.TEST_SIZE_IN_GB,
                                             run_as_root=True)
@@ -535,7 +547,7 @@ class NfsDriverTestCase(test.TestCase):
         self._set_driver()
         drv = self._driver
 
-        self.configuration.nfs_mount_point_base = self.TEST_MNT_POINT_BASE
+        self.override_config('nfs_mount_point_base', self.TEST_MNT_POINT_BASE)
 
         self.assertEqual('/mnt/test/2f4f60214cf43c595666dd815f0360a4',
                          drv._get_mount_point_for_share(self.TEST_NFS_EXPORT1))
@@ -543,8 +555,8 @@ class NfsDriverTestCase(test.TestCase):
     def test_get_mount_point_for_share_given_extra_slash_in_state_path(self):
         """_get_mount_point_for_share should calculate correct value."""
         # This test gets called with the extra slash
-        self.configuration.nfs_mount_point_base = (
-            self.TEST_MNT_POINT_BASE_EXTRA_SLASH)
+        self.override_config('nfs_mount_point_base',
+                             self.TEST_MNT_POINT_BASE_EXTRA_SLASH)
 
         # The driver gets called with the correct configuration and removes
         # the extra slash
@@ -774,7 +786,8 @@ class NfsDriverTestCase(test.TestCase):
 
             self.assertEqual(ret, 0.14)
 
-    def test_create_sparsed_volume(self):
+    @mock.patch('cinder.objects.volume.Volume.save')
+    def test_create_sparsed_volume(self, mock_save):
         self._set_driver()
         drv = self._driver
         volume = self._simple_volume()
@@ -791,13 +804,12 @@ class NfsDriverTestCase(test.TestCase):
                                                                  mock.ANY)
                 mock_set_rw_permissions.assert_called_once_with(mock.ANY)
 
-    def test_create_nonsparsed_volume(self):
+    @mock.patch('cinder.objects.volume.Volume.save')
+    def test_create_nonsparsed_volume(self, mock_save):
         self._set_driver()
         drv = self._driver
-        self.configuration.nfs_sparsed_volumes = False
-        volume = self._simple_volume()
-
         self.override_config('nfs_sparsed_volumes', False)
+        volume = self._simple_volume()
 
         with mock.patch.object(
                 drv, '_create_regular_file') as mock_create_regular_file:
@@ -914,7 +926,7 @@ class NfsDriverTestCase(test.TestCase):
 
     def test_get_volume_stats_with_non_zero_reserved_percentage(self):
         """get_volume_stats must fill the correct values."""
-        self.configuration.reserved_percentage = 10.0
+        self.override_config('reserved_percentage', 10.0)
         drv = nfs.NfsDriver(configuration=self.configuration)
 
         drv._mounted_shares = [self.TEST_NFS_EXPORT1, self.TEST_NFS_EXPORT2]
@@ -1040,7 +1052,9 @@ class NfsDriverTestCase(test.TestCase):
                                                        total_allocated,
                                                        requested_volume_size))
 
-    def test_extend_volume(self):
+    @ddt.data(None, 'raw', 'qcow2')
+    @mock.patch('cinder.objects.volume.Volume.get_by_id')
+    def test_extend_volume(self, file_format, mock_get):
         """Extend a volume by 1."""
         self._set_driver()
         drv = self._driver
@@ -1049,19 +1063,25 @@ class NfsDriverTestCase(test.TestCase):
             id='80ee16b6-75d2-4d54-9539-ffc1b4b0fb10',
             size=1,
             provider_location='nfs_share')
-        path = 'path'
+        if file_format:
+            volume.admin_metadata = {'format': file_format}
+        mock_get.return_value = volume
+        local_vol_dir = 'dir'
         newSize = volume['size'] + 1
 
         with mock.patch.object(image_utils, 'resize_image') as resize:
-            with mock.patch.object(drv, 'local_path', return_value=path):
+            with mock.patch.object(drv, '_local_volume_dir',
+                                   return_value=local_vol_dir):
                 with mock.patch.object(drv, '_is_share_eligible',
                                        return_value=True):
                     with mock.patch.object(drv, '_is_file_size_equal',
                                            return_value=True):
                         drv.extend_volume(volume, newSize)
 
+                        path = os.path.join(local_vol_dir, volume.name)
                         resize.assert_called_once_with(path, newSize,
-                                                       run_as_root=True)
+                                                       run_as_root=True,
+                                                       file_format=file_format)
 
     def test_extend_volume_attached_fail(self):
         """Extend a volume by 1."""
@@ -1085,7 +1105,8 @@ class NfsDriverTestCase(test.TestCase):
                         self.assertRaises(exception.ExtendVolumeError,
                                           drv.extend_volume, volume, newSize)
 
-    def test_extend_volume_failure(self):
+    @mock.patch('cinder.objects.volume.Volume.get_by_id')
+    def test_extend_volume_failure(self, mock_get):
         """Error during extend operation."""
         self._set_driver()
         drv = self._driver
@@ -1094,6 +1115,8 @@ class NfsDriverTestCase(test.TestCase):
             id='80ee16b6-75d2-4d54-9539-ffc1b4b0fb10',
             size=1,
             provider_location='nfs_share')
+        volume.admin_metadata = {'format': 'qcow2'}
+        mock_get.return_value = volume
 
         with mock.patch.object(image_utils, 'resize_image'):
             with mock.patch.object(drv, 'local_path', return_value='path'):
@@ -1128,26 +1151,30 @@ class NfsDriverTestCase(test.TestCase):
         self._set_driver()
         drv = self._driver
         path = 'fake/path'
+        volume_name = 'volume1'
         size = 2
         data = mock.MagicMock()
         data.virtual_size = size * units.Gi
 
-        with mock.patch.object(image_utils, 'qemu_img_info',
-                               return_value=data):
-            self.assertTrue(drv._is_file_size_equal(path, size))
+        with mock.patch.object(drv, '_qemu_img_info',
+                               return_value=data) as mock_qemu_img_info:
+            self.assertTrue(drv._is_file_size_equal(path, volume_name, size))
+            mock_qemu_img_info.assert_called_once_with(path, volume_name)
 
     def test_is_file_size_equal_false(self):
         """File sizes are not equal."""
         self._set_driver()
         drv = self._driver
         path = 'fake/path'
+        volume_name = 'volume1'
         size = 2
         data = mock.MagicMock()
         data.virtual_size = (size + 1) * units.Gi
 
-        with mock.patch.object(image_utils, 'qemu_img_info',
-                               return_value=data):
-            self.assertFalse(drv._is_file_size_equal(path, size))
+        with mock.patch.object(drv, '_qemu_img_info',
+                               return_value=data) as mock_qemu_img_info:
+            self.assertFalse(drv._is_file_size_equal(path, volume_name, size))
+            mock_qemu_img_info.assert_called_once_with(path, volume_name)
 
     @mock.patch.object(nfs, 'LOG')
     def test_set_nas_security_options_when_true(self, LOG):
@@ -1304,17 +1331,22 @@ class NfsDriverTestCase(test.TestCase):
             dest_volume = self._simple_volume()
             src_volume = self._simple_volume()
 
+        # snapshot img_info
         fake_snap = fake_snapshot.fake_snapshot_obj(self.context)
         fake_snap.volume = src_volume
-
         img_out = qemu_img_info % {'volid': src_volume.id,
                                    'snapid': fake_snap.id,
                                    'size_gb': src_volume.size,
                                    'size_b': src_volume.size * units.Gi}
+        img_info = imageutils.QemuImgInfo(img_out, format='json')
+
+        # backing file img_info
+        img_out = QEMU_IMG_INFO_OUT1 % {'volid': src_volume.id,
+                                        'size_b': src_volume.size * units.Gi}
+        bk_img_info = imageutils.QemuImgInfo(img_out, format='json')
 
-        img_info = imageutils.QemuImgInfo(img_out)
         mock_img_info = self.mock_object(image_utils, 'qemu_img_info')
-        mock_img_info.return_value = img_info
+        mock_img_info.side_effect = [img_info, bk_img_info]
         mock_convert_image = self.mock_object(image_utils, 'convert_image')
 
         vol_dir = os.path.join(self.TEST_MNT_POINT_BASE,
@@ -1338,29 +1370,39 @@ class NfsDriverTestCase(test.TestCase):
                                        dest_encryption_key_id)
 
         mock_read_info_file.assert_called_once_with(info_path)
-        mock_img_info.assert_called_once_with(snap_path,
-                                              force_share=True,
-                                              run_as_root=True)
+        snap_info_call = mock.call(snap_path,
+                                   force_share=True, run_as_root=True,
+                                   allow_qcow2_backing_file=True)
+        src_info_call = mock.call(src_vol_path,
+                                  force_share=True, run_as_root=True,
+                                  allow_qcow2_backing_file=True)
+        self.assertEqual(2, mock_img_info.call_count)
+        mock_img_info.assert_has_calls([snap_info_call, src_info_call])
         used_qcow = nfs_conf['nfs_qcow2_volumes']
         if encryption:
             mock_convert_image.assert_called_once_with(
                 src_vol_path, dest_vol_path, 'luks',
                 passphrase_file='/tmp/passfile',
                 run_as_root=True,
-                src_passphrase_file='/tmp/imgfile')
+                src_passphrase_file='/tmp/imgfile',
+                data=bk_img_info)
         else:
             mock_convert_image.assert_called_once_with(
                 src_vol_path, dest_vol_path, 'qcow2' if used_qcow else 'raw',
-                run_as_root=True)
+                run_as_root=True,
+                data=bk_img_info)
         mock_permission.assert_called_once_with(dest_vol_path)
 
     @ddt.data([NFS_CONFIG1, QEMU_IMG_INFO_OUT3, 'available'],
               [NFS_CONFIG2, QEMU_IMG_INFO_OUT4, 'backing-up'],
+              [NFS_CONFIG2, QEMU_IMG_INFO_OUT4, 'restoring'],
               [NFS_CONFIG3, QEMU_IMG_INFO_OUT3, 'available'],
-              [NFS_CONFIG4, QEMU_IMG_INFO_OUT4, 'backing-up'])
+              [NFS_CONFIG4, QEMU_IMG_INFO_OUT4, 'backing-up'],
+              [NFS_CONFIG4, QEMU_IMG_INFO_OUT4, 'restoring'])
     @ddt.unpack
+    @mock.patch('cinder.objects.volume.Volume.save')
     def test_create_volume_from_snapshot(self, nfs_conf, qemu_img_info,
-                                         snap_status):
+                                         snap_status, mock_save):
         self._set_driver(extra_confs=nfs_conf)
         drv = self._driver
 
@@ -1392,7 +1434,7 @@ class NfsDriverTestCase(test.TestCase):
                                    'snapid': fake_snap.id,
                                    'size_gb': src_volume.size,
                                    'size_b': src_volume.size * units.Gi}
-        img_info = imageutils.QemuImgInfo(img_out)
+        img_info = imageutils.QemuImgInfo(img_out, format='json')
         mock_img_info = self.mock_object(image_utils, 'qemu_img_info')
         mock_img_info.return_value = img_info
 
@@ -1416,12 +1458,12 @@ class NfsDriverTestCase(test.TestCase):
         used_qcow = nfs_conf['nfs_qcow2_volumes']
         mock_convert_image.assert_called_once_with(
             src_volume_path, new_volume_path, 'qcow2' if used_qcow else 'raw',
-            run_as_root=True)
+            run_as_root=True, data=img_info)
         mock_ensure.assert_called_once()
         mock_find_share.assert_called_once_with(new_volume)
 
     @ddt.data('error', 'creating', 'deleting', 'deleted', 'updating',
-              'error_deleting', 'unmanaging', 'restoring')
+              'error_deleting', 'unmanaging')
     def test_create_volume_from_snapshot_invalid_status(self, snap_status):
         """Expect an error when the snapshot's status is not 'available'."""
         self._set_driver()
@@ -1458,7 +1500,8 @@ class NfsDriverTestCase(test.TestCase):
         mock_img_utils = self.mock_object(image_utils, 'qemu_img_info')
         img_out = qemu_img_info % {'volid': volume.id, 'size_gb': volume.size,
                                    'size_b': volume.size * units.Gi}
-        mock_img_utils.return_value = imageutils.QemuImgInfo(img_out)
+        mock_img_utils.return_value = imageutils.QemuImgInfo(img_out,
+                                                             format='json')
         self.mock_object(drv, '_read_info_file',
                          return_value={'active': "volume-%s" % volume.id})
 
@@ -1466,7 +1509,8 @@ class NfsDriverTestCase(test.TestCase):
 
         mock_img_utils.assert_called_once_with(vol_path,
                                                force_share=True,
-                                               run_as_root=True)
+                                               run_as_root=True,
+                                               allow_qcow2_backing_file=True)
         self.assertEqual('nfs', conn_info['driver_volume_type'])
         self.assertEqual(volume.name, conn_info['data']['name'])
         self.assertEqual(self.TEST_MNT_POINT_BASE,
@@ -1478,23 +1522,46 @@ class NfsDriverTestCase(test.TestCase):
         drv = self._driver
         volume = self._simple_volume()
 
-        qemu_img_output = """image: %s
-        file format: iso
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        """ % volume['name']
-        mock_img_info.return_value = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "iso",
+    "virtual-size": 10737418240,
+    "actual-size": 173000
+}""" % volume['name']
+        mock_img_info.return_value = imageutils.QemuImgInfo(qemu_img_output,
+                                                            format='json')
 
-        self.assertRaises(exception.InvalidVolume,
-                          drv.initialize_connection,
-                          volume,
-                          None)
+        self.assertRaisesRegex(exception.InvalidVolume,
+                               "must be a valid raw or qcow2 image",
+                               drv.initialize_connection,
+                               volume,
+                               None)
+
+    @mock.patch.object(image_utils, 'qemu_img_info')
+    def test_initialize_connection_raise_on_wrong_size(self, mock_img_info):
+        self._set_driver()
+        drv = self._driver
+        volume = self._simple_volume()
+
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "qcow2",
+    "virtual-size": 999999999999999,
+    "actual-size": 173000
+}""" % volume['name']
+        mock_img_info.return_value = imageutils.QemuImgInfo(qemu_img_output,
+                                                            format='json')
+        self.assertRaisesRegex(exception.InvalidVolume,
+                               "virtual_size does not match",
+                               drv.initialize_connection,
+                               volume,
+                               None)
 
     def test_create_snapshot(self):
         self._set_driver()
         drv = self._driver
         volume = self._simple_volume()
-        self.configuration.nfs_snapshot_support = True
+        self.override_config('nfs_snapshot_support', True)
         fake_snap = fake_snapshot.fake_snapshot_obj(self.context)
         fake_snap.volume = volume
         vol_dir = os.path.join(self.TEST_MNT_POINT_BASE,
