@@ -158,7 +158,7 @@ class VolumeConnectionTestCase(base.BaseVolumeTestCase):
             mock.patch.object(cinder.tests.fake_driver.FakeLoggingVolumeDriver,
                               'initialize_connection') as driver_init:
             type_qos.return_value = dict(qos_specs=qos_values)
-            type_extra_specs.return_value = 'True'
+            type_extra_specs.return_value = {}
             driver_init.return_value = {'data': {}}
             mock_get_target.return_value = None
             qos_specs_expected = {'key1': 'value1',
@@ -228,7 +228,7 @@ class VolumeConnectionTestCase(base.BaseVolumeTestCase):
             mock.patch.object(cinder.tests.fake_driver.FakeLoggingVolumeDriver,
                               'initialize_connection') as driver_init:
             type_qos.return_value = dict(qos_specs=qos_values)
-            type_extra_specs.return_value = 'True'
+            type_extra_specs.return_value = {}
             driver_init.return_value = {'data': {}}
             mock_get_target.return_value = None
             qos_specs_expected = {'write_iops_sec': 90,
@@ -300,7 +300,7 @@ class VolumeConnectionTestCase(base.BaseVolumeTestCase):
             mock.patch.object(cinder.tests.fake_driver.FakeLoggingVolumeDriver,
                               'initialize_connection') as driver_init:
             type_qos.return_value = dict(qos_specs=qos_values)
-            type_extra_specs.return_value = 'True'
+            type_extra_specs.return_value = {}
             driver_init.return_value = {'data': {}}
             mock_get_target.return_value = None
             qos_specs_expected = {'write_iops_sec': 800,
@@ -372,7 +372,7 @@ class VolumeConnectionTestCase(base.BaseVolumeTestCase):
             mock.patch.object(cinder.tests.fake_driver.FakeLoggingVolumeDriver,
                               'initialize_connection') as driver_init:
             type_qos.return_value = dict(qos_specs=qos_values)
-            type_extra_specs.return_value = 'True'
+            type_extra_specs.return_value = {}
             driver_init.return_value = {'data': {}}
             mock_get_target.return_value = None
             qos_specs_expected = {'write_iops_sec': 3000,
