@@ -70,7 +70,9 @@ class SCISCSIDriver(storagecenter_common.SCCommonDriver,
     """
 
     VERSION = '4.1.2'
-    CI_WIKI_NAME = "Dell_EMC_SC_Series_CI"
+    CI_WIKI_NAME = "DellEMC_SC_CI"
+
+    SUPPORTED = False
 
     def __init__(self, *args, **kwargs):
         super(SCISCSIDriver, self).__init__(*args, **kwargs)
