@@ -24,6 +24,7 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import importutils
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder.objects import fields
@@ -44,8 +45,8 @@ else:
 
 LOG = logging.getLogger(__name__)
 
-PROTOCOL_FC = 'FC'
-PROTOCOL_ISCSI = 'iSCSI'
+PROTOCOL_FC = constants.FC
+PROTOCOL_ISCSI = constants.ISCSI
 
 
 class VolumeParams(object):
@@ -530,7 +531,7 @@ class CommonAdapter(object):
         }
         return conn_info
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def initialize_connection(self, volume, connector):
         lun = self.client.get_lun(lun_id=self.get_lun_id(volume))
         return self._initialize_connection(lun, connector, volume.id)
@@ -578,7 +579,7 @@ class CommonAdapter(object):
             'data': data,
         }
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def terminate_connection(self, volume, connector):
         lun = self.client.get_lun(lun_id=self.get_lun_id(volume))
         # None `connector` indicates force detach, then detach all even the
@@ -813,7 +814,9 @@ class CommonAdapter(object):
             is_compressed=vol_params.is_compressed)
         src_id = src_snap.get_id()
         try:
-            conn_props = cinder_utils.brick_get_connector_properties()
+            conn_props = volume_utils.brick_get_connector_properties(
+                self.config.use_multipath_for_image_xfer,
+                self.config.enforce_multipath_for_image_xfer)
 
             with self._connect_resource(dest_lun, conn_props,
                                         vol_params.volume_id) as dest_info, \
@@ -952,17 +955,17 @@ class CommonAdapter(object):
     def get_pool_id_by_name(self, name):
         return self.client.get_pool_id_by_name(name=name)
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def initialize_connection_snapshot(self, snapshot, connector):
         snap = self.client.get_snap(snapshot.name)
         return self._initialize_connection(snap, connector, snapshot.id)
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def terminate_connection_snapshot(self, snapshot, connector):
         snap = self.client.get_snap(snapshot.name)
         return self._terminate_connection(snap, connector)
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def restore_snapshot(self, volume, snapshot):
         return self.client.restore_snapshot(snapshot.name)
 
@@ -1265,7 +1268,7 @@ class CommonAdapter(object):
         """The failover only happens manually, no need to update the status."""
         return [], []
 
-    @cinder_utils.trace
+    @volume_utils.trace
     def failover(self, volumes, secondary_id=None, groups=None):
         # TODO(ryan) support group failover after group bp merges
         # https://review.opendev.org/#/c/574119/
