@@ -16,11 +16,9 @@
 import functools
 import gettext
 import inspect
-import platform
 
 from oslo_log import log as logging
 from oslo_utils import timeutils
-import six
 
 from cinder.i18n import _
 from cinder import version
@@ -240,12 +238,6 @@ class IBMStorageProxy(object):
                 'wwpns': wwpns,
                 'chap': chap}
 
-    @_trace_time
-    def _get_os_type(self):
-        """Gets a string representation of the current os"""
-        dist = platform.dist()
-        return "%s-%s-%s" % (dist[0], dist[1], platform.processor())
-
     def _log(self, level, message, **kwargs):
         """Wrapper around the logger"""
         to_log = _(self.prefix + message)  # NOQA
@@ -268,7 +260,7 @@ class IBMStorageProxy(object):
             return "Status: '%s', Code: %s" % (
                 exception.status, exception.code)
 
-        return six.text_type(exception)
+        return str(exception)
 
     def _get_driver_super(self):
         """Gets the IBM Storage Drivers super class
