@@ -20,7 +20,6 @@ from eventlet import greenthread
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -52,7 +51,7 @@ class StorwizeSVCReplication(object):
         except Exception as e:
             msg = (_('Unable to fail-over the volume %(id)s to the '
                    'secondary back-end, error: %(error)s') %
-                   {"id": vref['id'], "error": six.text_type(e)})
+                   {"id": vref['id'], "error": str(e)})
             LOG.exception(msg)
             raise exception.VolumeDriverException(message=msg)
 
@@ -68,7 +67,7 @@ class StorwizeSVCReplication(object):
             except Exception as e:
                 msg = (_('Unable to fail-back the volume: %(vol)s to the '
                          'master back-end, error: %(error)s') %
-                       {"vol": volume['name'], "error": six.text_type(e)})
+                       {"vol": volume['name'], "error": str(e)})
                 LOG.exception(msg)
                 raise exception.VolumeDriverException(message=msg)
 
@@ -98,20 +97,24 @@ class StorwizeSVCReplicationGlobalMirror(StorwizeSVCReplication):
 
         target_vol_name = storwize_const.REPLICA_AUX_VOL_PREFIX + vref['name']
         try:
-            attr = self.target_helpers.get_vdisk_attributes(target_vol_name)
-            if not attr:
-                opts = self.driver._get_vdisk_params(vref['volume_type_id'])
-                pool = self.target.get('pool_name')
-                src_attr = self.driver._helpers.get_vdisk_attributes(
-                    vref['name'])
-                opts['iogrp'] = src_attr['IO_group_id']
+            opts = self.driver._get_vdisk_params(vref['volume_type_id'])
+            pool = self.target.get('pool_name')
+            src_attr = self.driver._helpers.get_vdisk_attributes(
+                vref['name'])
+            opts['iogrp'] = src_attr['IO_group_id']
+            opts['mirror_pool'] = None
+            try:
                 self.target_helpers.create_vdisk(target_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb', pool, opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    LOG.info('Target Volume: %(vol)s already exists',
+                             {'vol': target_vol_name})
 
-            system_info = self.target_helpers.get_system_info()
+            target_system_id = self.driver._aux_state['system_id']
             self.driver._helpers.create_relationship(
-                vref['name'], target_vol_name, system_info.get('system_id'),
+                vref['name'], target_vol_name, target_system_id,
                 self.asyncmirror)
         except Exception as e:
             msg = (_("Unable to set up mirror mode replication for %(vol)s. "
@@ -184,7 +187,7 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
                 src_change_pool = src_child_pool
             try:
                 self.driver._helpers.create_vdisk(source_change_vol_name,
-                                                  six.text_type(vref['size']),
+                                                  str(vref['size']),
                                                   'gb',
                                                   src_change_pool,
                                                   src_change_opts)
@@ -201,7 +204,7 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
             target_opts['iogrp'] = src_attr['IO_group_id']
             try:
                 self.target_helpers.create_vdisk(target_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb',
                                                  target_pool,
                                                  target_opts)
@@ -228,7 +231,7 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
             target_change_opts['autoexpand'] = True
             try:
                 self.target_helpers.create_vdisk(target_change_vol_name,
-                                                 six.text_type(vref['size']),
+                                                 str(vref['size']),
                                                  'gb',
                                                  target_change_pool,
                                                  target_change_opts)
@@ -257,7 +260,7 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
         except Exception as e:
             msg = (_("Unable to set up gmcv mode replication for %(vol)s. "
                      "Exception: %(err)s.") % {'vol': vref['id'],
-                                               'err': six.text_type(e)})
+                                               'err': str(e)})
             LOG.exception(msg)
             raise exception.VolumeDriverException(message=msg)
         LOG.debug('leave: volume_replication_setup:volume %s', vref['name'])
@@ -303,7 +306,7 @@ class StorwizeSVCReplicationManager(object):
                         return processutils.ssh_execute(
                             ssh, command, check_exit_code=check_exit_code)
                     except Exception as e:
-                        LOG.error(six.text_type(e))
+                        LOG.error(str(e))
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
                 try:
