@@ -87,7 +87,8 @@ class NetAppBlockStorageLibrary(
                                  'xen', 'hyper_v']
     DEFAULT_LUN_OS = 'linux'
     DEFAULT_HOST_TYPE = 'linux'
-    DEFAULT_FILTER_FUNCTION = 'capabilities.utilization < 70'
+    DEFAULT_FILTER_FUNCTION = ('capabilities.utilization < 70 and '
+                               'capabilities.total_volumes < 1024')
     DEFAULT_GOODNESS_FUNCTION = '100 - capabilities.utilization'
 
     def __init__(self, driver_name, driver_protocol, **kwargs):
@@ -189,10 +190,13 @@ class NetAppBlockStorageLibrary(
         """
 
         # Add the task that deletes snapshots marked for deletion.
-        self.loopingcalls.add_task(
-            self._delete_snapshots_marked_for_deletion,
-            loopingcalls.ONE_MINUTE,
-            loopingcalls.ONE_MINUTE)
+        # ADD snapshot cleanup task to ASA r2 once snapshot feature is
+        # implemented in the driver.
+        if not self.configuration.netapp_disaggregated_platform:
+            self.loopingcalls.add_task(
+                self._delete_snapshots_marked_for_deletion,
+                loopingcalls.ONE_MINUTE,
+                loopingcalls.ONE_MINUTE)
 
         # Add the task that logs EMS messages
         self.loopingcalls.add_task(
