@@ -23,6 +23,7 @@
 """
 Volume driver library for NetApp C-mode block storage systems.
 """
+import time
 
 from oslo_log import log as logging
 from oslo_service import loopingcall
@@ -33,6 +34,7 @@ from cinder import exception
 from cinder.i18n import _
 from cinder.objects import fields
 from cinder.volume.drivers.netapp.dataontap import block_base
+from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
 from cinder.volume.drivers.netapp.dataontap.utils import capabilities
 from cinder.volume.drivers.netapp.dataontap.utils import data_motion
@@ -132,11 +134,18 @@ class NetAppBlockStorageCmodeLibrary(
 
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate."""
-        if not self._get_flexvol_to_pool_map():
+        if (not self._get_flexvol_to_pool_map()
+                and not self.configuration.netapp_disaggregated_platform):
             msg = _('No pools are available for provisioning volumes. '
                     'Ensure that the configuration option '
                     'netapp_pool_name_search_pattern is set correctly.')
             raise na_utils.NetAppDriverException(msg)
+        elif (self.configuration.netapp_disaggregated_platform
+              and not self._get_cluster_to_pool_map()):
+            msg = _('No pools are available for provisioning volumes. '
+                    'Ensure ASA r2 configuration option is set correctly.')
+            raise na_utils.NetAppDriverException(msg)
+
         self._add_looping_tasks()
         super(NetAppBlockStorageCmodeLibrary, self).check_for_setup_error()
 
@@ -235,14 +244,30 @@ class NetAppBlockStorageCmodeLibrary(
         metadata = self._get_lun_attr(name, 'metadata')
         volume = metadata['Volume']
 
-        self.zapi_client.clone_lun(
-            volume, name, new_name, space_reserved,
-            qos_policy_group_name=qos_policy_group_name,
-            src_block=src_block, dest_block=dest_block,
-            block_count=block_count,
-            source_snapshot=source_snapshot,
-            is_snapshot=is_snapshot,
-            qos_policy_group_is_adaptive=qos_policy_group_is_adaptive)
+        try:
+            self.zapi_client.clone_lun(
+                volume, name, new_name, space_reserved,
+                qos_policy_group_name=qos_policy_group_name,
+                src_block=src_block, dest_block=dest_block,
+                block_count=block_count,
+                source_snapshot=source_snapshot,
+                is_snapshot=is_snapshot,
+                qos_policy_group_is_adaptive=qos_policy_group_is_adaptive,
+            )
+        except netapp_api.NaApiError as e:
+            with excutils.save_and_reraise_exception() as exc_context:
+                if 'Device busy' in e.message:
+                    self._retry_clone_lun(
+                        volume, name, new_name, space_reserved,
+                        qos_policy_group_name=qos_policy_group_name,
+                        src_block=src_block, dest_block=dest_block,
+                        block_count=block_count,
+                        source_snapshot=source_snapshot,
+                        is_snapshot=is_snapshot,
+                        qos_policy_group_is_adaptive=(
+                            qos_policy_group_is_adaptive),
+                    )
+                    exc_context.reraise = False
 
         LOG.debug("Cloned LUN with new name %s", new_name)
         lun = self.zapi_client.get_lun_by_args(vserver=self.vserver,
@@ -260,6 +285,49 @@ class NetAppBlockStorageCmodeLibrary(
                                  clone_lun['Size'],
                                  clone_lun))
 
+    def _retry_clone_lun(self, volume, name, new_name, space_reserved,
+                         qos_policy_group_name=None, src_block=0,
+                         dest_block=0, block_count=0,
+                         source_snapshot=None, is_snapshot=False,
+                         qos_policy_group_is_adaptive=False):
+        """Retry lun clone creation when ONTAP throws device busy error"""
+        # timeout and interval are configurable parameters that the user can
+        # specify under the backend stanza. If the user does not set these
+        # values, default values will be used. For example, if timeout is set
+        # to 60 seconds and interval is set to 5 seconds, then this code will
+        # retry the LUN clone every 5 seconds until the 60-second timeout is
+        # reached.
+        timeout = self.configuration.safe_get('netapp_lun_clone_busy_timeout')
+        interval = self.configuration.safe_get(
+            'netapp_lun_clone_busy_interval')
+        retries = int(timeout / interval)
+
+        for attempt in range(1, retries + 1):
+            try:
+                self.zapi_client.clone_lun(
+                    volume, name, new_name, space_reserved,
+                    qos_policy_group_name=qos_policy_group_name,
+                    src_block=src_block, dest_block=dest_block,
+                    block_count=block_count,
+                    source_snapshot=source_snapshot,
+                    is_snapshot=is_snapshot,
+                    qos_policy_group_is_adaptive=qos_policy_group_is_adaptive,
+                )
+                LOG.info("LUN clone succeeded on attempt %s.", attempt)
+                break
+            except netapp_api.NaApiError as e:
+                if 'Device busy' in e.message:
+                    LOG.debug("Attempt %s failed with device busy error."
+                              "Retrying after %s seconds...", attempt,
+                              interval)
+                    if attempt == retries:
+                        msg = _("Timed out after %s retry for LUN clone"
+                                " creation")
+                        raise na_utils.NetAppDriverException(msg % retries)
+                    time.sleep(interval)
+                else:
+                    raise netapp_api.NaApiError(e.code, e.message)
+
     def _get_fc_target_wwpns(self, include_partner=True):
         return self.zapi_client.get_fc_target_wwpns()
 
@@ -300,7 +368,10 @@ class NetAppBlockStorageCmodeLibrary(
 
         # Utilization and performance metrics require cluster-scoped
         # credentials
-        if self.using_cluster_credentials:
+        # Performance metrics are skipped for disaggregated for now.
+        # TODO(jayaanan): Add support for performance metrics for ASA r2
+        if (self.using_cluster_credentials
+                and not self.configuration.netapp_disaggregated_platform):
             # Get up-to-date node utilization metrics just once
             self.perf_library.update_performance_cache(ssc)
 
@@ -329,8 +400,11 @@ class NetAppBlockStorageCmodeLibrary(
                 self.max_over_subscription_ratio)
 
             # Add up-to-date capacity info
-            capacity = self.zapi_client.get_flexvol_capacity(
-                flexvol_name=ssc_vol_name)
+            if self.configuration.netapp_disaggregated_platform:
+                capacity = self.zapi_client.get_cluster_capacity()
+            else:
+                capacity = self.zapi_client.get_flexvol_capacity(
+                    flexvol_name=ssc_vol_name)
 
             size_total_gb = capacity['size-total'] / units.Gi
             pool['total_capacity_gb'] = na_utils.round_down(size_total_gb)
@@ -338,9 +412,10 @@ class NetAppBlockStorageCmodeLibrary(
             size_available_gb = capacity['size-available'] / units.Gi
             pool['free_capacity_gb'] = na_utils.round_down(size_available_gb)
 
+            luns = self.zapi_client.get_lun_sizes_by_volume(
+                ssc_vol_name)
+            pool['total_volumes'] = len(luns)
             if self.configuration.netapp_driver_reports_provisioned_capacity:
-                luns = self.zapi_client.get_lun_sizes_by_volume(
-                    ssc_vol_name)
                 provisioned_cap = 0
                 for lun in luns:
                     lun_name = lun['path'].split('/')[-1]
@@ -351,9 +426,12 @@ class NetAppBlockStorageCmodeLibrary(
                 pool['provisioned_capacity_gb'] = na_utils.round_down(
                     float(provisioned_cap) / units.Gi)
 
-            if self.using_cluster_credentials:
-                dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
-                    ssc_vol_name)
+            if (self.using_cluster_credentials and
+                    not self.configuration.netapp_disaggregated_platform):
+                dedupe_used = (
+                    self.zapi_client
+                    .get_flexvol_dedupe_used_percent(ssc_vol_name)
+                )
             else:
                 dedupe_used = 0.0
             pool['netapp_dedupe_used_percent'] = na_utils.round_down(
@@ -381,8 +459,10 @@ class NetAppBlockStorageCmodeLibrary(
 
     def _update_ssc(self):
         """Refresh the storage service catalog with the latest set of pools."""
-
-        self.ssc_library.update_ssc(self._get_flexvol_to_pool_map())
+        if self.configuration.netapp_disaggregated_platform:
+            self.ssc_library.update_ssc_asa(self._get_cluster_to_pool_map())
+        else:
+            self.ssc_library.update_ssc(self._get_flexvol_to_pool_map())
 
     def _get_flexvol_to_pool_map(self):
         """Get the flexvols that match the pool name search pattern.
@@ -413,6 +493,9 @@ class NetAppBlockStorageCmodeLibrary(
 
         return pools
 
+    def _get_cluster_to_pool_map(self):
+        return dot_utils.get_cluster_to_pool_map(self.zapi_client)
+
     def delete_volume(self, volume):
         """Driver entry point for destroying existing volumes."""
         super(NetAppBlockStorageCmodeLibrary, self).delete_volume(volume)
