@@ -24,6 +24,7 @@ from oslo_utils import units
 from cinder import coordination
 from cinder import exception
 from cinder.i18n import _
+from cinder.objects import fields
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
 from cinder.volume.drivers.netapp.dataontap.utils import capabilities
@@ -72,7 +73,8 @@ class NetAppNVMeStorageLibrary(
     ALLOWED_SUBSYSTEM_HOST_TYPES = ['aix', 'linux', 'vmware', 'windows']
     DEFAULT_NAMESPACE_OS = 'linux'
     DEFAULT_HOST_TYPE = 'linux'
-    DEFAULT_FILTER_FUNCTION = 'capabilities.utilization < 70'
+    DEFAULT_FILTER_FUNCTION = 'capabilities.utilization < 70 and ' \
+                              'capabilities.total_volumes < 1024'
     DEFAULT_GOODNESS_FUNCTION = '100 - capabilities.utilization'
     REQUIRED_CMODE_FLAGS = ['netapp_vserver']
     NVME_PORT = 4420
@@ -149,7 +151,11 @@ class NetAppNVMeStorageLibrary(
     def _update_ssc(self):
         """Refresh the storage service catalog with the latest set of pools."""
 
-        self.ssc_library.update_ssc(self._get_flexvol_to_pool_map())
+        """Refresh the storage service catalog with the latest set of pools."""
+        if self.configuration.netapp_disaggregated_platform:
+            self.ssc_library.update_ssc_asa(self._get_cluster_to_pool_map())
+        else:
+            self.ssc_library.update_ssc(self._get_flexvol_to_pool_map())
 
     def _get_flexvol_to_pool_map(self):
         """Get the flexvols that match the pool name search pattern.
@@ -180,16 +186,25 @@ class NetAppNVMeStorageLibrary(
 
         return pools
 
+    def _get_cluster_to_pool_map(self):
+        return dot_utils.get_cluster_to_pool_map(self.client)
+
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate.
 
         Discovers the namespaces on the NetApp server.
         """
-        if not self._get_flexvol_to_pool_map():
+        if (not self.configuration.netapp_disaggregated_platform
+                and not self._get_flexvol_to_pool_map()):
             msg = _('No pools are available for provisioning volumes. '
                     'Ensure that the configuration option '
                     'netapp_pool_name_search_pattern is set correctly.')
             raise na_utils.NetAppDriverException(msg)
+        elif self.configuration.netapp_disaggregated_platform:
+            if not self._get_cluster_to_pool_map():
+                msg = _('No pools are available for provisioning volumes. '
+                        'Ensure ASA r2 configuration option is set correctly.')
+                raise na_utils.NetAppDriverException(msg)
         self._add_looping_tasks()
 
         if self.namespace_ostype not in self.ALLOWED_NAMESPACE_OS_TYPES:
@@ -413,7 +428,11 @@ class NetAppNVMeStorageLibrary(
         """Extracts the namespace from API and populates the table."""
 
         path = namespace['Path']
-        (_rest, _splitter, name) = path.rpartition('/')
+        if self.configuration.netapp_disaggregated_platform:
+            name = path
+        else:
+            (_rest, _splitter, name) = path.rpartition('/')
+
         handle = self._create_namespace_handle(namespace)
         size = namespace['Size']
         return NetAppNamespace(handle, name, size, namespace)
@@ -504,7 +523,8 @@ class NetAppNVMeStorageLibrary(
 
         # Utilization and performance metrics require cluster-scoped
         # credentials
-        if self.using_cluster_credentials:
+        if (self.using_cluster_credentials
+                and not self.configuration.netapp_disaggregated_platform):
             # Get up-to-date node utilization metrics just once
             self.perf_library.update_performance_cache(ssc)
 
@@ -524,17 +544,20 @@ class NetAppNVMeStorageLibrary(
 
             # Add driver capabilities and config info
             pool['QoS_support'] = False
-            pool['multiattach'] = False
-            pool['online_extend_support'] = False
-            pool['consistencygroup_support'] = False
-            pool['consistent_group_snapshot_enabled'] = False
+            pool['multiattach'] = True
+            pool['online_extend_support'] = True
+            pool['consistencygroup_support'] = True
+            pool['consistent_group_snapshot_enabled'] = True
             pool['reserved_percentage'] = self.reserved_percentage
             pool['max_over_subscription_ratio'] = (
                 self.max_over_subscription_ratio)
 
             # Add up-to-date capacity info
-            capacity = self.client.get_flexvol_capacity(
-                flexvol_name=ssc_vol_name)
+            if self.configuration.netapp_disaggregated_platform:
+                capacity = self.client.get_cluster_capacity()
+            else:
+                capacity = self.client.get_flexvol_capacity(
+                    flexvol_name=ssc_vol_name)
 
             size_total_gb = capacity['size-total'] / units.Gi
             pool['total_capacity_gb'] = na_utils.round_down(size_total_gb)
@@ -542,9 +565,10 @@ class NetAppNVMeStorageLibrary(
             size_available_gb = capacity['size-available'] / units.Gi
             pool['free_capacity_gb'] = na_utils.round_down(size_available_gb)
 
+            namespaces = self.client.get_namespace_sizes_by_volume(
+                ssc_vol_name)
+            pool['total_volumes'] = len(namespaces)
             if self.configuration.netapp_driver_reports_provisioned_capacity:
-                namespaces = self.client.get_namespace_sizes_by_volume(
-                    ssc_vol_name)
                 provisioned_cap = 0
                 for namespace in namespaces:
                     namespace_name = namespace['path'].split('/')[-1]
@@ -556,7 +580,8 @@ class NetAppNVMeStorageLibrary(
                 pool['provisioned_capacity_gb'] = na_utils.round_down(
                     float(provisioned_cap) / units.Gi)
 
-            if self.using_cluster_credentials:
+            if (self.using_cluster_credentials and
+                    not self.configuration.netapp_disaggregated_platform):
                 dedupe_used = self.client.get_flexvol_dedupe_used_percent(
                     ssc_vol_name)
             else:
@@ -610,67 +635,46 @@ class NetAppNVMeStorageLibrary(
 
         self.namespace_table[name].size = new_size_bytes
 
-    def _get_or_create_subsystem(self, host_nqn, host_os_type):
-        """Checks for an subsystem for a host.
-
-        Creates subsystem if not already present with given host os type and
-        adds the host.
-        """
-        # Backend supports different subsystems with the same hosts, so
-        # instead of reusing non OpenStack subsystem, we make sure we only use
-        # our own, thus being compatible with custom subsystem.
-        subsystems = self.client.get_subsystem_by_host(
-            host_nqn)
-        if subsystems:
-            subsystem_name = subsystems[0]['name']
-            host_os_type = subsystems[0]['os_type']
-        else:
-            subsystem_name = na_utils.OPENSTACK_PREFIX + str(uuid.uuid4())
-            self.client.create_subsystem(subsystem_name, host_os_type,
-                                         host_nqn)
-
-        return subsystem_name, host_os_type
-
     def _find_mapped_namespace_subsystem(self, path, host_nqn):
         """Find an subsystem for a namespace mapped to the given host."""
         subsystems = [subsystem['name'] for subsystem in
                       self.client.get_subsystem_by_host(host_nqn)]
 
         # Map subsystem name to namespace-id for the requested host.
-        namespace_map = {v['subsystem']: v['uuid']
+        namespace_map = {v['uuid']: (v['subsystem_uuid'], v['subsystem'])
                          for v in self.client.get_namespace_map(path)
                          if v['subsystem'] in subsystems}
 
-        subsystem_name = n_uuid = None
+        subsystem_uuid = subsystem_name = n_uuid = None
         # Give preference to OpenStack subsystems, just use the last one if not
         # present to allow unmapping old mappings that used a custom subsystem.
-        for subsystem_name, n_uuid in namespace_map.items():
+        for n_uuid, (subsystem_uuid, subsystem_name) in namespace_map.items():
             if subsystem_name.startswith(na_utils.OPENSTACK_PREFIX):
                 break
 
-        return subsystem_name, n_uuid
+        return subsystem_uuid, subsystem_name, n_uuid
 
     def _map_namespace(self, name, host_nqn):
         """Maps namespace to the host nqn and returns its ID assigned."""
-
-        subsystem_name, subsystem_host_os = self._get_or_create_subsystem(
-            host_nqn, self.host_type)
-        if subsystem_host_os != self.host_type:
-            LOG.warning("Namespace misalignment may occur for current"
-                        " subsystem %(sub_name)s with host OS type"
-                        " %(sub_os)s. Please configure subsystem manually"
-                        " according to the type of the host OS.",
-                        {'sub_name': subsystem_name,
-                         'sub_os': subsystem_host_os})
-
         metadata = self._get_namespace_attr(name, 'metadata')
         path = metadata['Path']
         try:
-            ns_uuid = self.client.map_namespace(
-                path, subsystem_name,)
+            subsystems = self.client.get_namespace_map(path)
+            ns_uuid = subsystem_uuid = None
+            if subsystems:
+                subsystem_name = subsystems[0]['subsystem']
+                subsystem_uuid = subsystems[0]['subsystem_uuid']
+                ns_uuid = subsystems[0]['uuid']
+                self.client.map_host_with_subsystem(host_nqn, subsystem_uuid)
+            else:
+                subsystem_name = na_utils.OPENSTACK_PREFIX + str(uuid.uuid4())
+                self.client.create_subsystem(subsystem_name, self.host_type,
+                                             host_nqn)
+                ns_uuid = self.client.map_namespace(path, subsystem_name, )
             return subsystem_name, ns_uuid
         except netapp_api.NaApiError as e:
-            (subsystem_name, ns_uuid) = self._find_mapped_namespace_subsystem(
+            (_, subsystem_name, ns_uuid) =\
+                self._find_mapped_namespace_subsystem(
                 path, host_nqn)
             if ns_uuid is not None and subsystem_name:
                 return subsystem_name, ns_uuid
@@ -737,18 +741,18 @@ class NetAppNVMeStorageLibrary(
     def _unmap_namespace(self, path, host_nqn):
         """Unmaps a namespace from given host."""
 
-        namespace_unmap_list = []
-        if host_nqn:
-            (subsystem, _) = self._find_mapped_namespace_subsystem(
-                path, host_nqn)
-            namespace_unmap_list.append((path, subsystem))
-        else:
-            namespace_maps = self.client.get_namespace_map(path)
-            namespace_unmap_list = [
-                (path, m['subsystem']) for m in namespace_maps]
+        if not host_nqn:
+            LOG.warning("Nothing to unmap - host_nqn is missing: %s", path)
+            return
 
-        for _path, _subsystem in namespace_unmap_list:
-            self.client.unmap_namespace(_path, _subsystem)
+        (subsystem_uuid, _, _) = self._find_mapped_namespace_subsystem(
+            path, host_nqn)
+
+        if subsystem_uuid:
+            self.client.unmap_host_with_subsystem(host_nqn, subsystem_uuid)
+        else:
+            LOG.debug("No mapping exists between namespace: %s"
+                      " and host_nqn: %s", path, host_nqn)
 
     @coordination.synchronized('netapp-terminate-nvme-connection-{volume.id}')
     def terminate_connection(self, volume, connector, **kwargs):
@@ -776,3 +780,151 @@ class NetAppNVMeStorageLibrary(
         metadata = self._get_namespace_attr(name, 'metadata')
         path = metadata['Path']
         self._unmap_namespace(path, host_nqn)
+
+    def create_group(self, group):
+        """Driver entry point for creating a generic volume group.
+
+        ONTAP does not maintain an actual Group construct. As a result, no
+        communication to the backend is necessary for generic volume group
+        creation.
+
+        :returns: Hard-coded model update for generic volume group model.
+        """
+        model_update = {'status': fields.GroupStatus.AVAILABLE}
+        return model_update
+
+    def delete_group(self, group, volumes):
+        """Driver entry point for deleting a group.
+
+        :returns: Updated group model and list of volume models
+                 for the volumes that were deleted.
+        """
+        model_update = {'status': fields.GroupStatus.DELETED}
+        volumes_model_update = []
+        for volume in volumes:
+            try:
+                self.delete_volume(volume)
+                volumes_model_update.append(
+                    {'id': volume['id'], 'status': 'deleted'})
+            except Exception:
+                volumes_model_update.append(
+                    {'id': volume['id'],
+                     'status': 'error_deleting'})
+                LOG.exception("Volume %(vol)s in the group could not be "
+                              "deleted.", {'vol': volume})
+        return model_update, volumes_model_update
+
+    def update_group(self, group, add_volumes=None, remove_volumes=None):
+        """Driver entry point for updating a generic volume group.
+
+        Since no actual group construct is ever created in ONTAP, it is not
+        necessary to update any metadata on the backend. Since this is a NO-OP,
+        there is guaranteed to be no change in any of the volumes' statuses.
+        """
+        return None, None, None
+
+    def create_group_snapshot(self, group_snapshot, snapshots):
+        """Creates a Cinder group snapshot object.
+
+        The Cinder group snapshot object is created by making use of an
+        ephemeral ONTAP consistency group snapshot in order to provide
+        write-order consistency for a set of flexvol snapshots. First, a list
+        of the flexvols backing the given Cinder group must be gathered. An
+        ONTAP group-snapshot of these flexvols will create a snapshot copy of
+        all the Cinder volumes in the generic volume group. For each Cinder
+        volume in the group, it is then necessary to clone its backing
+        namespace from the ONTAP cg-snapshot. The naming convention used for
+        the clones is what indicates the clone's role as a Cinder snapshot
+        and its inclusion in a Cinder group. The ONTAP cg-snapshot of the
+        flexvols is no longer required after having cloned the namespaces
+        backing the Cinder volumes in the Cinder group.
+
+        :returns: An implicit update for group snapshot and snapshots models
+                 that is interpreted by the manager to set their models to
+                 available.
+        """
+        try:
+            if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
+                self._create_consistent_group_snapshot(group_snapshot,
+                                                       snapshots)
+            else:
+                for snapshot in snapshots:
+                    self._create_snapshot(snapshot)
+        except Exception as ex:
+            err_msg = (_("Create group snapshot failed (%s).") % ex)
+            LOG.exception(err_msg, resource=group_snapshot)
+            raise na_utils.NetAppDriverException(err_msg)
+
+        return None, None
+
+    def _create_consistent_group_snapshot(self, group_snapshot, snapshots):
+        flexvols = set()
+        for snapshot in snapshots:
+            flexvols.add(volume_utils.extract_host(
+                snapshot['volume']['host'], level='pool'))
+
+        self.client.create_cg_snapshot(flexvols, group_snapshot['id'])
+
+        for snapshot in snapshots:
+            self._clone_namespace(snapshot['volume']['name'], snapshot['name'])
+
+        for flexvol in flexvols:
+            try:
+                self.client.wait_for_busy_snapshot(
+                    flexvol, group_snapshot['id'])
+                self.client.delete_snapshot(
+                    flexvol, group_snapshot['id'])
+            except exception.SnapshotIsBusy:
+                self.client.mark_snapshot_for_deletion(
+                    flexvol, group_snapshot['id'])
+
+    def delete_group_snapshot(self, group_snapshot, snapshots):
+        """Delete namespaces backing each snapshot in the group snapshot.
+
+        :returns: An implicit update for snapshots models that is interpreted
+                 by the manager to set their models to delete.
+        """
+        for snapshot in snapshots:
+            self._delete_namespace(snapshot['name'])
+            LOG.debug("Snapshot %s deletion successful", snapshot['name'])
+
+        return None, None
+
+    def create_group_from_src(self, group, volumes, group_snapshot=None,
+                              snapshots=None, source_group=None,
+                              source_vols=None):
+        """Creates a group from a group snapshot or a group of cinder vols.
+
+        :returns: An implicit update for the volumes model that is
+                 interpreted by the manager as a successful operation.
+        """
+        LOG.debug("VOLUMES %s ", ', '.join([vol['id'] for vol in volumes]))
+        volume_model_updates = []
+
+        if group_snapshot:
+            vols = zip(volumes, snapshots)
+
+            for volume, snapshot in vols:
+                source = {
+                    'name': snapshot['name'],
+                    'size': snapshot['volume_size'],
+                }
+                self._clone_source_to_destination(source, volume)
+                '''if volume_model_update is not None:
+                    volume_model_update['id'] = volume['id']
+                    volume_model_updates.append(volume_model_update)'''
+
+        else:
+            vols = zip(volumes, source_vols)
+
+            for volume, old_src_vref in vols:
+                src_namespace = self._get_namespace_from_table(
+                    old_src_vref['name'])
+                source = {'name': src_namespace.name,
+                          'size': old_src_vref['size']}
+                self._clone_source_to_destination(source, volume)
+                '''if volume_model_update is not None:
+                    volume_model_update['id'] = volume['id']
+                    volume_model_updates.append(volume_model_update)'''
+
+        return None, volume_model_updates
