@@ -193,7 +193,14 @@ netapp_cluster_opts = [
     cfg.StrOpt('netapp_vserver',
                help=('This option specifies the virtual storage server '
                      '(Vserver) name on the storage cluster on which '
-                     'provisioning of block storage volumes should occur.')), ]
+                     'provisioning of block storage volumes should occur.')),
+    cfg.BoolOpt('netapp_disaggregated_platform',
+                default=False,
+                help=('This option specifies whether to enable ASA r2 '
+                      'workflows for NetApp disaggregated platform. '
+                      'When set to True, the driver will use ASA r2 '
+                      'specific client and workflows for interacting '
+                      'with NetApp ONTAP.')), ]
 
 netapp_img_cache_opts = [
     cfg.IntOpt('netapp_nfs_image_cache_cleanup_interval',
@@ -257,7 +264,19 @@ netapp_san_opts = [
                      'applied to the names of objects from the storage '
                      'backend which represent pools in Cinder. This option '
                      'is only utilized when the storage protocol is '
-                     'configured to use iSCSI or FC.')), ]
+                     'configured to use iSCSI or FC.')),
+    cfg.IntOpt('netapp_lun_clone_busy_timeout',
+               min=0,
+               default=30,
+               help='Specifies the maximum time (in seconds) to retry'
+                    ' the LUN clone operation when an ONTAP "device busy"'
+                    ' error occurs.'),
+    cfg.IntOpt('netapp_lun_clone_busy_interval',
+               min=0,
+               default=3,
+               help='Specifies the time interval (in seconds) to retry'
+                    ' the LUN clone operation when an ONTAP "device busy"'
+                    ' error occurs.')]
 
 netapp_replication_opts = [
     cfg.MultiOpt('netapp_replication_aggregate_map',
