@@ -30,11 +30,11 @@ import re
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 from oslo_utils import netutils
-import six
 
 from cinder import context
 from cinder import exception
 from cinder.i18n import _
+from cinder.objects import fields
 from cinder import utils
 from cinder import version
 from cinder.volume import qos_specs
@@ -51,7 +51,33 @@ DEPRECATED_SSC_SPECS = {'netapp_unmirrored': 'netapp_mirrored',
                         'netapp_nodedup': 'netapp_dedup',
                         'netapp_nocompression': 'netapp_compression',
                         'netapp_thick_provisioned': 'netapp_thin_provisioned'}
-QOS_KEYS = frozenset(['maxIOPS', 'maxIOPSperGiB', 'maxBPS', 'maxBPSperGiB'])
+MIN_QOS_KEYS = frozenset([
+    'minIOPS',
+    'minIOPSperGiB',
+])
+MAX_QOS_KEYS = frozenset([
+    'maxIOPS',
+    'maxIOPSperGiB',
+    'maxBPS',
+    'maxBPSperGiB',
+])
+ADAPTIVE_QOS_KEYS = frozenset([
+    'expectedIOPSperGiB',
+    'peakIOPSperGiB',
+    'expectedIOPSAllocation',
+    'peakIOPSAllocation',
+    'absoluteMinIOPS',
+    'blockSize',
+])
+QOS_ADAPTIVE_POLICY_GROUP_SPEC_KEYS = frozenset([
+    'expected_iops',
+    'peak_iops',
+    'expected_iops_allocation',
+    'peak_iops_allocation',
+    'absolute_min_iops',
+    'block_size',
+    'policy_name',
+])
 BACKEND_QOS_CONSUMERS = frozenset(['back-end', 'both'])
 
 # Secret length cannot be less than 96 bits. http://tools.ietf.org/html/rfc3723
@@ -64,6 +90,14 @@ class NetAppDriverException(exception.VolumeDriverException):
     message = _("NetApp Cinder Driver exception.")
 
 
+class GeometryHasChangedOnDestination(NetAppDriverException):
+    message = _("Geometry has changed on destination volume.")
+
+
+class NetAppDriverTimeout(NetAppDriverException):
+    message = _("Timeout in NetApp Cinder Driver.")
+
+
 def validate_instantiation(**kwargs):
     """Checks if a driver is instantiated other than by the unified driver.
 
@@ -87,7 +121,7 @@ def check_flags(required_flags, configuration):
 def to_bool(val):
     """Converts true, yes, y, 1 to True, False otherwise."""
     if val:
-        strg = six.text_type(val).lower()
+        strg = str(val).lower()
         if (strg == 'true' or strg == 'y'
             or strg == 'yes' or strg == 'enabled'
                 or strg == '1'):
@@ -151,8 +185,15 @@ def trace_filter_func_api(all_args):
     return re.match(API_TRACE_PATTERN, api_name) is not None
 
 
+def trace_filter_func_rest_api(all_args):
+    url = all_args.get('url')
+    if url is None:
+        return True
+    return re.match(API_TRACE_PATTERN, url) is not None
+
+
 def round_down(value, precision='0.00'):
-    return float(decimal.Decimal(six.text_type(value)).quantize(
+    return float(decimal.Decimal(str(value)).quantize(
         decimal.Decimal(precision), rounding=decimal.ROUND_DOWN))
 
 
@@ -176,7 +217,7 @@ def get_iscsi_connection_properties(lun_id, volume, iqns,
                  for a in addresses]
 
     lun_id = int(lun_id)
-    if isinstance(iqns, six.string_types):
+    if isinstance(iqns, str):
         iqns = [iqns] * len(addresses)
 
     target_portals = ['%s:%s' % (a, p) for a, p in zip(addresses, ports)]
@@ -208,17 +249,37 @@ def validate_qos_spec(qos_spec):
     """Check validity of Cinder qos spec for our backend."""
     if qos_spec is None:
         return
-    normalized_qos_keys = [key.lower() for key in QOS_KEYS]
-    keylist = []
-    for key, value in qos_spec.items():
-        lower_case_key = key.lower()
-        if lower_case_key not in normalized_qos_keys:
-            msg = _('Unrecognized QOS keyword: "%s"') % key
-            raise exception.Invalid(msg)
-        keylist.append(lower_case_key)
-    # Modify the following check when we allow multiple settings in one spec.
-    if len(keylist) > 1:
-        msg = _('Only one limit can be set in a QoS spec.')
+
+    normalized_min_keys = [key.lower() for key in MIN_QOS_KEYS]
+    normalized_max_keys = [key.lower() for key in MAX_QOS_KEYS]
+    normalized_aqos_keys = [key.lower() for key in ADAPTIVE_QOS_KEYS]
+
+    unrecognized_keys = [
+        k for k in qos_spec.keys()
+        if k.lower() not in
+        normalized_max_keys + normalized_min_keys + normalized_aqos_keys]
+
+    if unrecognized_keys:
+        msg = _('Unrecognized QOS keywords: "%s"') % unrecognized_keys
+        raise exception.Invalid(msg)
+
+    min_dict = {k: v for k, v in qos_spec.items()
+                if k.lower() in normalized_min_keys}
+    if len(min_dict) > 1:
+        msg = _('Only one minimum limit can be set in a QoS spec.')
+        raise exception.Invalid(msg)
+
+    max_dict = {k: v for k, v in qos_spec.items()
+                if k.lower() in normalized_max_keys}
+    if len(max_dict) > 1:
+        msg = _('Only one maximum limit can be set in a QoS spec.')
+        raise exception.Invalid(msg)
+
+    aqos_dict = {k: v for k, v in qos_spec.items()
+                 if k.lower() in normalized_aqos_keys}
+    if aqos_dict and (min_dict or max_dict):
+        msg = _('Adaptive QoS specs and non-adaptive QoS specs '
+                'cannot be used together.')
         raise exception.Invalid(msg)
 
 
@@ -231,26 +292,101 @@ def get_volume_type_from_volume(volume):
     return volume_types.get_volume_type(ctxt, type_id)
 
 
+def _get_min_throughput_from_qos_spec(qos_spec, volume_size):
+    """Returns the minimum QoS throughput.
+
+    The QoS min specs are exclusive of one another and it accepts values in
+    IOPS only.
+    """
+    if 'miniops' in qos_spec:
+        min_throughput = '%siops' % qos_spec['miniops']
+    elif 'miniopspergib' in qos_spec:
+        min_throughput = '%siops' % str(
+            int(qos_spec['miniopspergib']) * int(volume_size))
+    else:
+        min_throughput = None
+    return min_throughput
+
+
+def _get_max_throughput_from_qos_spec(qos_spec, volume_size):
+    """Returns the maximum QoS throughput.
+
+    The QoS max specs are exclusive of one another.
+    """
+    if 'maxiops' in qos_spec:
+        max_throughput = '%siops' % qos_spec['maxiops']
+    elif 'maxiopspergib' in qos_spec:
+        max_throughput = '%siops' % str(
+            int(qos_spec['maxiopspergib']) * int(volume_size))
+    elif 'maxbps' in qos_spec:
+        max_throughput = '%sB/s' % qos_spec['maxbps']
+    elif 'maxbpspergib' in qos_spec:
+        max_throughput = '%sB/s' % str(
+            int(qos_spec['maxbpspergib']) * int(volume_size))
+    else:
+        max_throughput = None
+    return max_throughput
+
+
 def map_qos_spec(qos_spec, volume):
     """Map Cinder QOS spec to limit/throughput-value as used in client API."""
     if qos_spec is None:
         return None
 
-    qos_spec = map_dict_to_lower(qos_spec)
-    spec = dict(policy_name=get_qos_policy_group_name(volume),
-                max_throughput=None)
+    spec = map_dict_to_lower(qos_spec)
+    min_throughput = _get_min_throughput_from_qos_spec(spec, volume['size'])
+    max_throughput = _get_max_throughput_from_qos_spec(spec, volume['size'])
 
-    # QoS specs are exclusive of one another.
-    if 'maxiops' in qos_spec:
-        spec['max_throughput'] = '%siops' % qos_spec['maxiops']
-    elif 'maxiopspergib' in qos_spec:
-        spec['max_throughput'] = '%siops' % six.text_type(
-            int(qos_spec['maxiopspergib']) * int(volume['size']))
-    elif 'maxbps' in qos_spec:
-        spec['max_throughput'] = '%sB/s' % qos_spec['maxbps']
-    elif 'maxbpspergib' in qos_spec:
-        spec['max_throughput'] = '%sB/s' % six.text_type(
-            int(qos_spec['maxbpspergib']) * int(volume['size']))
+    if min_throughput and max_throughput and max_throughput.endswith('B/s'):
+        msg = _('Maximum limit should be in IOPS when minimum limit is '
+                'specified.')
+        raise exception.Invalid(msg)
+
+    if min_throughput and max_throughput and max_throughput < min_throughput:
+        msg = _('Maximum limit should be greater than or equal to the '
+                'minimum limit.')
+        raise exception.Invalid(msg)
+
+    policy = dict(policy_name=get_qos_policy_group_name(volume))
+    if min_throughput:
+        policy['min_throughput'] = min_throughput
+    if max_throughput:
+        policy['max_throughput'] = max_throughput
+    return policy
+
+
+def map_aqos_spec(qos_spec, volume):
+    """Map Cinder QOS spec to Adaptive QoS values."""
+    if qos_spec is None:
+        return None
+
+    qos_spec = map_dict_to_lower(qos_spec)
+    spec = dict(policy_name=get_qos_policy_group_name(volume))
+
+    # Adaptive QoS specs
+    if 'expectediopspergib' in qos_spec:
+        spec['expected_iops'] = (
+            '%sIOPS/GB' % qos_spec['expectediopspergib'])
+    if 'peakiopspergib' in qos_spec:
+        spec['peak_iops'] = '%sIOPS/GB' % qos_spec['peakiopspergib']
+    if 'expectediopsallocation' in qos_spec:
+        spec['expected_iops_allocation'] = qos_spec['expectediopsallocation']
+    if 'peakiopsallocation' in qos_spec:
+        spec['peak_iops_allocation'] = qos_spec['peakiopsallocation']
+    if 'absoluteminiops' in qos_spec:
+        spec['absolute_min_iops'] = '%sIOPS' % qos_spec['absoluteminiops']
+    if 'blocksize' in qos_spec:
+        spec['block_size'] = qos_spec['blocksize']
+
+    if 'peak_iops' not in spec or 'expected_iops' not in spec:
+        msg = _('Adaptive QoS requires the expected property and '
+                'the peak property set together.')
+        raise exception.Invalid(msg)
+
+    if spec['peak_iops'] < spec['expected_iops']:
+        msg = _('Adaptive maximum limit should be greater than or equal to '
+                'the adaptive minimum limit.')
+        raise exception.Invalid(msg)
 
     return spec
 
@@ -266,7 +402,7 @@ def map_dict_to_lower(input_dict):
 def get_qos_policy_group_name(volume):
     """Return the name of backend QOS policy group based on its volume id."""
     if 'id' in volume:
-        return OPENSTACK_PREFIX + volume['id']
+        return OPENSTACK_PREFIX + volume.name_id
     return None
 
 
@@ -313,7 +449,7 @@ def get_valid_qos_policy_group_info(volume, extra_specs=None):
     info = dict(legacy=None, spec=None)
     try:
         volume_type = get_volume_type_from_volume(volume)
-    except KeyError:
+    except (KeyError, exception.NotFound):
         LOG.exception('Cannot get QoS spec for volume %s.', volume['id'])
         return info
     if volume_type is None:
@@ -331,11 +467,35 @@ def get_valid_qos_policy_group_info(volume, extra_specs=None):
 
 def get_valid_backend_qos_spec_from_volume_type(volume, volume_type):
     """Given a volume type, return the associated Cinder QoS spec."""
-    spec_key_values = get_backend_qos_spec_from_volume_type(volume_type)
-    if spec_key_values is None:
+    spec_dict = get_backend_qos_spec_from_volume_type(volume_type)
+    if spec_dict is None:
         return None
-    validate_qos_spec(spec_key_values)
-    return map_qos_spec(spec_key_values, volume)
+    validate_qos_spec(spec_dict)
+    map_spec = (map_aqos_spec
+                if is_qos_adaptive(spec_dict)
+                else map_qos_spec)
+    return map_spec(spec_dict, volume)
+
+
+def is_qos_adaptive(spec_dict):
+    if not spec_dict:
+        return False
+
+    normalized_aqos_keys = [key.lower() for key in ADAPTIVE_QOS_KEYS]
+    return all(key in normalized_aqos_keys
+               for key in map_dict_to_lower(spec_dict).keys())
+
+
+def is_qos_policy_group_spec_adaptive(policy):
+    if not policy:
+        return False
+
+    spec = policy.get('spec')
+    if not spec:
+        return False
+
+    return all(key in QOS_ADAPTIVE_POLICY_GROUP_SPEC_KEYS
+               for key in map_dict_to_lower(spec).keys())
 
 
 def get_backend_qos_spec_from_volume_type(volume_type):
@@ -350,8 +510,7 @@ def get_backend_qos_spec_from_volume_type(volume_type):
     # Front end QoS specs are handled by libvirt and we ignore them here.
     if consumer not in BACKEND_QOS_CONSUMERS:
         return None
-    spec_key_values = qos_spec['specs']
-    return spec_key_values
+    return qos_spec['specs']
 
 
 def check_for_invalid_qos_spec_combination(info, volume_type):
@@ -394,6 +553,33 @@ def get_export_host_junction_path(share):
     return host, junction_path
 
 
+def qos_min_feature_name(is_nfs, node_name):
+    if node_name is None:
+        node_name = ''
+    if is_nfs:
+        return 'QOS_MIN_NFS_' + node_name
+    else:
+        return 'QOS_MIN_BLOCK_' + node_name
+
+
+def is_multiattach_to_host(volume, connector):
+    # With multi-attach enabled, a single volume can be attached to multiple
+    # instances. If multiple instances are running on the same nova host, the
+    # volume should remain attached to the nova host until it is detached
+    # from the last instance on that host.
+
+    if not volume.multiattach or not volume.volume_attachment:
+        return False
+    attachment = [
+        attach_info
+        for attach_info in volume.volume_attachment
+        if attach_info['attach_status'] == fields.VolumeAttachStatus.ATTACHED
+        and attach_info['attached_host'] == connector.get('host')
+    ]
+    LOG.debug('is_multiattach_to_host: attachment %s.', attachment)
+    return len(attachment) > 1
+
+
 class hashabledict(dict):
     """A hashable dictionary that is comparable (i.e. in unit tests, etc.)"""
     def __hash__(self):
