@@ -15,11 +15,13 @@
 
 import base64
 import functools
+import hashlib
 import json
 import math
 from os import urandom
 from random import randint
 import re
+import urllib
 
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.asymmetric import padding
@@ -31,11 +33,8 @@ import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
-from oslo_utils.secretutils import md5
 from oslo_utils import units
 import requests
-from six.moves import urllib
-from six import string_types
 
 from cinder import exception
 from cinder.i18n import _
@@ -118,7 +117,7 @@ class AESCipher(object):
         d = d_i = b''
         while len(d) < key_length + iv_length:
             md5_str = d_i + password + salt
-            d_i = md5(md5_str, usedforsecurity=True).digest()
+            d_i = hashlib.md5(md5_str, usedforsecurity=True).digest()
             d += d_i
         return d[:key_length], d[key_length:key_length + iv_length]
 
@@ -456,7 +455,7 @@ class SynoCommon(object):
                 or 0 >= len(out['data']['nodes'])
                 or not self.check_value_valid(out['data']['nodes'][0],
                                               ['uuid'],
-                                              string_types)):
+                                              str)):
             msg = _('Failed to _get_node_uuid.')
             raise exception.VolumeDriverException(message=msg)
 
@@ -565,7 +564,7 @@ class SynoCommon(object):
             with excutils.save_and_reraise_exception():
                 LOG.exception('Failed to _get_lun_uuid. [%s]', lun_name)
 
-        if not self.check_value_valid(lun_info, ['uuid'], string_types):
+        if not self.check_value_valid(lun_info, ['uuid'], str):
             raise exception.MalformedResponse(cmd='_get_lun_uuid',
                                               reason=_('uuid not found'))
 
@@ -583,7 +582,7 @@ class SynoCommon(object):
             with excutils.save_and_reraise_exception():
                 LOG.exception('Failed to _get_lun_status. [%s]', lun_name)
 
-        if not self.check_value_valid(lun_info, ['status'], string_types):
+        if not self.check_value_valid(lun_info, ['status'], str):
             raise exception.MalformedResponse(cmd='_get_lun_status',
                                               reason=_('status not found'))
         if not self.check_value_valid(lun_info, ['is_action_locked'], bool):
@@ -635,7 +634,7 @@ class SynoCommon(object):
                 LOG.exception('Failed to _get_snapshot_info. [%s]',
                               snapshot_uuid)
 
-        if not self.check_value_valid(snapshot_info, ['status'], string_types):
+        if not self.check_value_valid(snapshot_info, ['status'], str):
             raise exception.MalformedResponse(cmd='_get_snapshot_status',
                                               reason=_('status not found'))
         if not self.check_value_valid(snapshot_info,
@@ -869,7 +868,7 @@ class SynoCommon(object):
 
         if not self.check_value_valid(out,
                                       ['data', 'firmware_ver'],
-                                      string_types):
+                                      str):
             raise exception.MalformedResponse(cmd='_check_ds_version',
                                               reason=_('data not found'))
         firmware_version = out['data']['firmware_ver']
@@ -993,7 +992,7 @@ class SynoCommon(object):
     def get_provider_location(self, iqn, trg_id):
         portals = ['%(ip)s:%(port)d' % {'ip': self.get_ip(),
                                         'port': self.target_port}]
-        sec_ips = self.config.safe_get('iscsi_secondary_ip_addresses')
+        sec_ips = self.config.safe_get('target_secondary_ip_addresses')
         for ip in sec_ips:
             portals.append('%(ip)s:%(port)d' %
                            {'ip': ip,
@@ -1180,7 +1179,7 @@ class SynoCommon(object):
 
         if not self.check_value_valid(resp,
                                       ['data', 'snapshot_uuid'],
-                                      string_types):
+                                      str):
             raise exception.MalformedResponse(cmd='create_snapshot',
                                               reason=_('uuid not found'))
 
@@ -1288,7 +1287,7 @@ class SynoCommon(object):
             'access_mode': 'rw',
             'discard': False
         }
-        ips = self.config.safe_get('iscsi_secondary_ip_addresses')
+        ips = self.config.safe_get('target_secondary_ip_addresses')
         if ips:
             target_portals = [iscsi_properties['target_portal']]
             for ip in ips:
